@@ -29,7 +29,6 @@
 #endif
 
 #include "cogl-debug.h"
-#include "cogl-internal.h"
 #include "cogl-util.h"
 #include "cogl-texture-private.h"
 #include "cogl-atlas-texture-private.h"
@@ -44,6 +43,8 @@
 #include "cogl-atlas.h"
 #include "cogl1-context.h"
 #include "cogl-sub-texture.h"
+#include "cogl-error-private.h"
+#include "cogl-texture-gl-private.h"
 
 #include <stdlib.h>
 
@@ -57,9 +58,9 @@ static CoglSubTexture *
 _cogl_atlas_texture_create_sub_texture (CoglTexture *full_texture,
                                         const CoglRectangleMapEntry *rectangle)
 {
+  CoglContext *ctx = full_texture->context;
   /* Create a subtexture for the given rectangle not including the
      1-pixel border */
-  _COGL_GET_CONTEXT (ctx, NULL);
   return cogl_sub_texture_new (ctx,
                                full_texture,
                                rectangle->x + 1,
@@ -191,17 +192,13 @@ _cogl_atlas_texture_atlas_destroyed_cb (void *user_data)
 }
 
 static CoglAtlas *
-_cogl_atlas_texture_create_atlas (void)
+_cogl_atlas_texture_create_atlas (CoglContext *ctx)
 {
   static CoglUserDataKey atlas_private_key;
 
-  CoglAtlas *atlas;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
-
-  atlas = _cogl_atlas_new (COGL_PIXEL_FORMAT_RGBA_8888,
-                           0,
-                           _cogl_atlas_texture_update_position_cb);
+  CoglAtlas *atlas = _cogl_atlas_new (COGL_PIXEL_FORMAT_RGBA_8888,
+                                      0,
+                                      _cogl_atlas_texture_update_position_cb);
 
   _cogl_atlas_add_reorganize_callback (atlas,
                                        _cogl_atlas_texture_pre_reorganize_cb,
@@ -247,18 +244,18 @@ _cogl_atlas_texture_foreach_sub_texture_in_region (
 }
 
 static void
-_cogl_atlas_texture_set_wrap_mode_parameters (CoglTexture *tex,
-                                              GLenum wrap_mode_s,
-                                              GLenum wrap_mode_t,
-                                              GLenum wrap_mode_p)
+_cogl_atlas_texture_gl_flush_legacy_texobj_wrap_modes (CoglTexture *tex,
+                                                       GLenum wrap_mode_s,
+                                                       GLenum wrap_mode_t,
+                                                       GLenum wrap_mode_p)
 {
   CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
 
   /* Forward on to the sub texture */
-  _cogl_texture_set_wrap_mode_parameters (atlas_tex->sub_texture,
-                                          wrap_mode_s,
-                                          wrap_mode_t,
-                                          wrap_mode_p);
+  _cogl_texture_gl_flush_legacy_texobj_wrap_modes (atlas_tex->sub_texture,
+                                                   wrap_mode_s,
+                                                   wrap_mode_t,
+                                                   wrap_mode_p);
 }
 
 static void
@@ -348,59 +345,69 @@ _cogl_atlas_texture_get_gl_texture (CoglTexture *tex,
 }
 
 static void
-_cogl_atlas_texture_set_filters (CoglTexture *tex,
-                                 GLenum min_filter,
-                                 GLenum mag_filter)
+_cogl_atlas_texture_gl_flush_legacy_texobj_filters (CoglTexture *tex,
+                                                    GLenum min_filter,
+                                                    GLenum mag_filter)
 {
   CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
 
   /* Forward on to the sub texture */
-  _cogl_texture_set_filters (atlas_tex->sub_texture, min_filter, mag_filter);
+  _cogl_texture_gl_flush_legacy_texobj_filters (atlas_tex->sub_texture,
+                                                min_filter, mag_filter);
 }
 
 static void
 _cogl_atlas_texture_migrate_out_of_atlas (CoglAtlasTexture *atlas_tex)
 {
+  CoglTexture *standalone_tex;
+
   /* Make sure this texture is not in the atlas */
-  if (atlas_tex->atlas)
-    {
-      CoglTexture *standalone_tex;
+  if (!atlas_tex->atlas)
+    return;
 
-      COGL_NOTE (ATLAS, "Migrating texture out of the atlas");
+  COGL_NOTE (ATLAS, "Migrating texture out of the atlas");
 
-      /* We don't know if any journal entries currently depend on
-       * OpenGL texture coordinates that would be invalidated by
-       * migrating textures in this atlas so we flush all journals
-       * before migrating.
-       *
-       * We are assuming that texture atlas migration never happens
-       * during a flush so we don't have to consider recursion here.
-       */
-      cogl_flush ();
+  /* We don't know if any journal entries currently depend on
+   * OpenGL texture coordinates that would be invalidated by
+   * migrating textures in this atlas so we flush all journals
+   * before migrating.
+   *
+   * We are assuming that texture atlas migration never happens
+   * during a flush so we don't have to consider recursion here.
+   */
+  cogl_flush ();
 
-      standalone_tex =
-        _cogl_atlas_copy_rectangle (atlas_tex->atlas,
-                                    atlas_tex->rectangle.x + 1,
-                                    atlas_tex->rectangle.y + 1,
-                                    atlas_tex->rectangle.width - 2,
-                                    atlas_tex->rectangle.height - 2,
-                                    COGL_TEXTURE_NO_ATLAS,
-                                    atlas_tex->format);
+  standalone_tex =
+    _cogl_atlas_copy_rectangle (atlas_tex->atlas,
+                                atlas_tex->rectangle.x + 1,
+                                atlas_tex->rectangle.y + 1,
+                                atlas_tex->rectangle.width - 2,
+                                atlas_tex->rectangle.height - 2,
+                                COGL_TEXTURE_NO_ATLAS,
+                                atlas_tex->format);
+  /* Note: we simply silently ignore failures to migrate a texture
+   * out (most likely due to lack of memory) and hope for the
+   * best.
+   *
+   * Maybe we should find a way to report the problem back to the
+   * app.
+   */
+  if (!standalone_tex)
+    return;
 
-      /* Notify cogl-pipeline.c that the texture's underlying GL texture
-       * storage is changing so it knows it may need to bind a new texture
-       * if the CoglTexture is reused with the same texture unit. */
-      _cogl_pipeline_texture_storage_change_notify (COGL_TEXTURE (atlas_tex));
+  /* Notify cogl-pipeline.c that the texture's underlying GL texture
+   * storage is changing so it knows it may need to bind a new texture
+   * if the CoglTexture is reused with the same texture unit. */
+  _cogl_pipeline_texture_storage_change_notify (COGL_TEXTURE (atlas_tex));
 
-      /* We need to unref the sub texture after doing the copy because
-         the copy can involve rendering which might cause the texture
-         to be used if it is used from a layer that is left in a
-         texture unit */
-      cogl_object_unref (atlas_tex->sub_texture);
-      atlas_tex->sub_texture = standalone_tex;
+  /* We need to unref the sub texture after doing the copy because
+     the copy can involve rendering which might cause the texture
+     to be used if it is used from a layer that is left in a
+     texture unit */
+  cogl_object_unref (atlas_tex->sub_texture);
+  atlas_tex->sub_texture = standalone_tex;
 
-      _cogl_atlas_texture_remove_from_atlas (atlas_tex);
-    }
+  _cogl_atlas_texture_remove_from_atlas (atlas_tex);
 }
 
 static void
@@ -432,63 +439,74 @@ _cogl_atlas_texture_ensure_non_quad_rendering (CoglTexture *tex)
 
 static CoglBool
 _cogl_atlas_texture_set_region_with_border (CoglAtlasTexture *atlas_tex,
-                                            int             src_x,
-                                            int             src_y,
-                                            int             dst_x,
-                                            int             dst_y,
-                                            unsigned int    dst_width,
-                                            unsigned int    dst_height,
-                                            CoglBitmap     *bmp)
+                                            int src_x,
+                                            int src_y,
+                                            int dst_x,
+                                            int dst_y,
+                                            int dst_width,
+                                            int dst_height,
+                                            CoglBitmap *bmp,
+                                            CoglError **error)
 {
   CoglAtlas *atlas = atlas_tex->atlas;
 
   /* Copy the central data */
-  if (!cogl_texture_set_region_from_bitmap (atlas->texture,
-                                            src_x, src_y,
-                                            dst_x + atlas_tex->rectangle.x + 1,
-                                            dst_y + atlas_tex->rectangle.y + 1,
-                                            dst_width,
-                                            dst_height,
-                                            bmp))
+  if (!_cogl_texture_set_region_from_bitmap (atlas->texture,
+                                             src_x, src_y,
+                                             dst_width,
+                                             dst_height,
+                                             bmp,
+                                             dst_x + atlas_tex->rectangle.x + 1,
+                                             dst_y + atlas_tex->rectangle.y + 1,
+                                             0, /* level 0 */
+                                             error))
     return FALSE;
 
   /* Update the left edge pixels */
   if (dst_x == 0 &&
-      !cogl_texture_set_region_from_bitmap (atlas->texture,
-                                            src_x, src_y,
-                                            atlas_tex->rectangle.x,
-                                            dst_y + atlas_tex->rectangle.y + 1,
-                                            1, dst_height,
-                                            bmp))
+      !_cogl_texture_set_region_from_bitmap (atlas->texture,
+                                             src_x, src_y,
+                                             1, dst_height,
+                                             bmp,
+                                             atlas_tex->rectangle.x,
+                                             dst_y + atlas_tex->rectangle.y + 1,
+                                             0, /* level 0 */
+                                             error))
     return FALSE;
   /* Update the right edge pixels */
   if (dst_x + dst_width == atlas_tex->rectangle.width - 2 &&
-      !cogl_texture_set_region_from_bitmap (atlas->texture,
-                                            src_x + dst_width - 1, src_y,
-                                            atlas_tex->rectangle.x +
-                                            atlas_tex->rectangle.width - 1,
-                                            dst_y + atlas_tex->rectangle.y + 1,
-                                            1, dst_height,
-                                            bmp))
+      !_cogl_texture_set_region_from_bitmap (atlas->texture,
+                                             src_x + dst_width - 1, src_y,
+                                             1, dst_height,
+                                             bmp,
+                                             atlas_tex->rectangle.x +
+                                             atlas_tex->rectangle.width - 1,
+                                             dst_y + atlas_tex->rectangle.y + 1,
+                                             0, /* level 0 */
+                                             error))
     return FALSE;
   /* Update the top edge pixels */
   if (dst_y == 0 &&
-      !cogl_texture_set_region_from_bitmap (atlas->texture,
-                                            src_x, src_y,
-                                            dst_x + atlas_tex->rectangle.x + 1,
-                                            atlas_tex->rectangle.y,
-                                            dst_width, 1,
-                                            bmp))
+      !_cogl_texture_set_region_from_bitmap (atlas->texture,
+                                             src_x, src_y,
+                                             dst_width, 1,
+                                             bmp,
+                                             dst_x + atlas_tex->rectangle.x + 1,
+                                             atlas_tex->rectangle.y,
+                                             0, /* level 0 */
+                                             error))
     return FALSE;
   /* Update the bottom edge pixels */
   if (dst_y + dst_height == atlas_tex->rectangle.height - 2 &&
-      !cogl_texture_set_region_from_bitmap (atlas->texture,
-                                            src_x, src_y + dst_height - 1,
-                                            dst_x + atlas_tex->rectangle.x + 1,
-                                            atlas_tex->rectangle.y +
-                                            atlas_tex->rectangle.height - 1,
-                                            dst_width, 1,
-                                            bmp))
+      !_cogl_texture_set_region_from_bitmap (atlas->texture,
+                                             src_x, src_y + dst_height - 1,
+                                             dst_width, 1,
+                                             bmp,
+                                             dst_x + atlas_tex->rectangle.x + 1,
+                                             atlas_tex->rectangle.y +
+                                             atlas_tex->rectangle.height - 1,
+                                             0, /* level 0 */
+                                             error))
     return FALSE;
 
   return TRUE;
@@ -496,7 +514,8 @@ _cogl_atlas_texture_set_region_with_border (CoglAtlasTexture *atlas_tex,
 
 static CoglBitmap *
 _cogl_atlas_texture_prepare_for_upload (CoglAtlasTexture *atlas_tex,
-                                        CoglBitmap *bmp)
+                                        CoglBitmap *bmp,
+                                        CoglError **error)
 {
   CoglPixelFormat internal_format;
   CoglBitmap *converted_bmp;
@@ -518,7 +537,8 @@ _cogl_atlas_texture_prepare_for_upload (CoglAtlasTexture *atlas_tex,
                                                     NULL, /* dst_format_out */
                                                     NULL, /* glintformat */
                                                     NULL, /* glformat */
-                                                    NULL /* gltype */);
+                                                    NULL, /* gltype */
+                                                    error);
 
   if (converted_bmp == NULL)
     return NULL;
@@ -541,17 +561,22 @@ _cogl_atlas_texture_prepare_for_upload (CoglAtlasTexture *atlas_tex,
 }
 
 static CoglBool
-_cogl_atlas_texture_set_region (CoglTexture    *tex,
-                                int             src_x,
-                                int             src_y,
-                                int             dst_x,
-                                int             dst_y,
-                                unsigned int    dst_width,
-                                unsigned int    dst_height,
-                                CoglBitmap     *bmp)
+_cogl_atlas_texture_set_region (CoglTexture *tex,
+                                int src_x,
+                                int src_y,
+                                int dst_x,
+                                int dst_y,
+                                int dst_width,
+                                int dst_height,
+                                int level,
+                                CoglBitmap *bmp,
+                                CoglError **error)
 {
   CoglAtlasTexture  *atlas_tex = COGL_ATLAS_TEXTURE (tex);
 
+  if (level != 0 && atlas_tex->atlas)
+    _cogl_atlas_texture_migrate_out_of_atlas (atlas_tex);
+
   /* If the texture is in the atlas then we need to copy the edge
      pixels to the border */
   if (atlas_tex->atlas)
@@ -559,14 +584,18 @@ _cogl_atlas_texture_set_region (CoglTexture    *tex,
       CoglBool ret;
 
       bmp = _cogl_atlas_texture_prepare_for_upload (atlas_tex,
-                                                    bmp);
+                                                    bmp,
+                                                    error);
+      if (!bmp)
+        return FALSE;
 
       /* Upload the data ignoring the premult bit */
       ret = _cogl_atlas_texture_set_region_with_border (atlas_tex,
                                                         src_x, src_y,
                                                         dst_x, dst_y,
                                                         dst_width, dst_height,
-                                                        bmp);
+                                                        bmp,
+                                                        error);
 
       cogl_object_unref (bmp);
 
@@ -574,11 +603,13 @@ _cogl_atlas_texture_set_region (CoglTexture    *tex,
     }
   else
     /* Otherwise we can just forward on to the sub texture */
-    return cogl_texture_set_region_from_bitmap (atlas_tex->sub_texture,
-                                                src_x, src_y,
-                                                dst_x, dst_y,
-                                                dst_width, dst_height,
-                                                bmp);
+    return _cogl_texture_set_region_from_bitmap (atlas_tex->sub_texture,
+                                                 src_x, src_y,
+                                                 dst_width, dst_height,
+                                                 bmp,
+                                                 dst_x, dst_y,
+                                                 level,
+                                                 error);
 }
 
 static CoglPixelFormat
@@ -598,25 +629,7 @@ _cogl_atlas_texture_get_gl_format (CoglTexture *tex)
   CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
 
   /* Forward on to the sub texture */
-  return _cogl_texture_get_gl_format (atlas_tex->sub_texture);
-}
-
-static int
-_cogl_atlas_texture_get_width (CoglTexture *tex)
-{
-  CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
-
-  /* Forward on to the sub texture */
-  return cogl_texture_get_width (atlas_tex->sub_texture);
-}
-
-static int
-_cogl_atlas_texture_get_height (CoglTexture *tex)
-{
-  CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
-
-  /* Forward on to the sub texture */
-  return cogl_texture_get_height (atlas_tex->sub_texture);
+  return _cogl_texture_gl_get_format (atlas_tex->sub_texture);
 }
 
 static CoglBool
@@ -634,37 +647,64 @@ _cogl_atlas_texture_can_use_format (CoglPixelFormat format)
 }
 
 CoglAtlasTexture *
-_cogl_atlas_texture_new_with_size (unsigned int width,
-                                   unsigned int height,
+_cogl_atlas_texture_new_with_size (CoglContext *ctx,
+                                   int width,
+                                   int height,
                                    CoglTextureFlags flags,
-                                   CoglPixelFormat internal_format)
+                                   CoglPixelFormat internal_format,
+                                   CoglError **error)
 {
   CoglAtlasTexture *atlas_tex;
-  CoglAtlas        *atlas;
-  GSList           *l;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
 
   /* Don't put textures in the atlas if the user has explicitly
      requested to disable it */
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_ATLAS)))
-    return NULL;
+    {
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "Atlasing disabled");
+      return NULL;
+    }
 
   /* We can't put the texture in the atlas if there are any special
      flags. This precludes textures with COGL_TEXTURE_NO_ATLAS and
      COGL_TEXTURE_NO_SLICING from being atlased */
   if (flags)
-    return NULL;
+    {
+      /* XXX: This is a bit of an odd error; if we make this api
+       * public then this should probably be dealt with at a higher
+       * level, in cogl-auto-texture.c:cogl_texture_new_with_size().
+       */
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "Usage constraints preclude atlasing texture");
+      return NULL;
+    }
 
   /* We can't atlas zero-sized textures because it breaks the atlas
      data structure */
   if (width < 1 || height < 1)
-    return NULL;
+    {
+      _cogl_set_error (error,
+                       COGL_TEXTURE_ERROR,
+                       COGL_TEXTURE_ERROR_SIZE,
+                       "1x1 atlas textures not supported");
+      return NULL;
+    }
 
   /* If we can't use FBOs then it will be too slow to migrate textures
      and we shouldn't use the atlas */
   if (!cogl_has_feature (ctx, COGL_FEATURE_ID_OFFSCREEN))
-    return NULL;
+    {
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "Atlasing disabled because migrations "
+                       "would be too slow");
+      return NULL;
+    }
 
   COGL_NOTE (ATLAS, "Adding texture of size %ix%i", width, height);
 
@@ -673,7 +713,10 @@ _cogl_atlas_texture_new_with_size (unsigned int width,
     {
       COGL_NOTE (ATLAS, "Texture can not be added because the "
                  "format is unsupported");
-
+      _cogl_set_error (error,
+                       COGL_TEXTURE_ERROR,
+                       COGL_TEXTURE_ERROR_FORMAT,
+                       "Texture format unsuitable for atlasing");
       return NULL;
     }
 
@@ -686,16 +729,32 @@ _cogl_atlas_texture_new_with_size (unsigned int width,
 
   _cogl_texture_init (COGL_TEXTURE (atlas_tex),
                       ctx,
+                      width, height,
                       &cogl_atlas_texture_vtable);
 
   atlas_tex->sub_texture = NULL;
 
+  atlas_tex->format = internal_format;
+  atlas_tex->atlas = NULL;
+
+  return _cogl_atlas_texture_object_new (atlas_tex);
+}
+
+static CoglBool
+_cogl_atlas_texture_allocate (CoglTexture *tex,
+                              CoglError **error)
+{
+  CoglContext *ctx = tex->context;
+  CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
+  CoglAtlas *atlas;
+  GSList *l;
+
   /* Look for an existing atlas that can hold the texture */
   for (l = ctx->atlases; l; l = l->next)
     /* Try to make some space in the atlas for the texture */
     if (_cogl_atlas_reserve_space (atlas = l->data,
                                    /* Add two pixels for the border */
-                                   width + 2, height + 2,
+                                   tex->width + 2, tex->height + 2,
                                    atlas_tex))
       {
         cogl_object_ref (atlas);
@@ -705,31 +764,36 @@ _cogl_atlas_texture_new_with_size (unsigned int width,
   /* If we couldn't find a suitable atlas then start another */
   if (l == NULL)
     {
-      atlas = _cogl_atlas_texture_create_atlas ();
+      atlas = _cogl_atlas_texture_create_atlas (ctx);
       COGL_NOTE (ATLAS, "Created new atlas for textures: %p", atlas);
       if (!_cogl_atlas_reserve_space (atlas,
                                       /* Add two pixels for the border */
-                                      width + 2, height + 2,
+                                      tex->width + 2, tex->height + 2,
                                       atlas_tex))
         {
           /* Ok, this means we really can't add it to the atlas */
           cogl_object_unref (atlas);
-          g_free (atlas_tex);
-          return NULL;
+
+          _cogl_set_error (error,
+                           COGL_SYSTEM_ERROR,
+                           COGL_SYSTEM_ERROR_NO_MEMORY,
+                           "Not enough memory to atlas texture");
+          return FALSE;
         }
     }
 
-  atlas_tex->format = internal_format;
   atlas_tex->atlas = atlas;
 
-  return _cogl_atlas_texture_object_new (atlas_tex);
+  return TRUE;
 }
 
 CoglAtlasTexture *
-_cogl_atlas_texture_new_from_bitmap (CoglBitmap      *bmp,
+_cogl_atlas_texture_new_from_bitmap (CoglBitmap *bmp,
                                      CoglTextureFlags flags,
-                                     CoglPixelFormat  internal_format)
+                                     CoglPixelFormat internal_format,
+                                     CoglError **error)
 {
+  CoglContext *ctx = _cogl_bitmap_get_context (bmp);
   CoglAtlasTexture *atlas_tex;
   CoglBitmap *dst_bmp;
   int bmp_width;
@@ -745,15 +809,22 @@ _cogl_atlas_texture_new_from_bitmap (CoglBitmap      *bmp,
   internal_format = _cogl_texture_determine_internal_format (bmp_format,
                                                              internal_format);
 
-  atlas_tex = _cogl_atlas_texture_new_with_size (bmp_width, bmp_height,
-                                                 flags, internal_format);
-
-  if (atlas_tex == NULL)
+  atlas_tex = _cogl_atlas_texture_new_with_size (ctx,
+                                                 bmp_width, bmp_height,
+                                                 flags, internal_format,
+                                                 error);
+  if (!atlas_tex)
     return NULL;
 
-  dst_bmp = _cogl_atlas_texture_prepare_for_upload (atlas_tex,
-                                                    bmp);
+  if (!cogl_texture_allocate (COGL_TEXTURE (atlas_tex), error))
+    {
+      cogl_object_unref (atlas_tex);
+      return NULL;
+    }
 
+  dst_bmp = _cogl_atlas_texture_prepare_for_upload (atlas_tex,
+                                                    bmp,
+                                                    error);
   if (dst_bmp == NULL)
     {
       cogl_object_unref (atlas_tex);
@@ -762,14 +833,20 @@ _cogl_atlas_texture_new_from_bitmap (CoglBitmap      *bmp,
 
   /* Defer to set_region so that we can share the code for copying the
      edge pixels to the border. */
-  _cogl_atlas_texture_set_region_with_border (atlas_tex,
-                                              0, /* src_x */
-                                              0, /* src_y */
-                                              0, /* dst_x */
-                                              0, /* dst_y */
-                                              bmp_width, /* dst_width */
-                                              bmp_height, /* dst_height */
-                                              dst_bmp);
+  if (!_cogl_atlas_texture_set_region_with_border (atlas_tex,
+                                                   0, /* src_x */
+                                                   0, /* src_y */
+                                                   0, /* dst_x */
+                                                   0, /* dst_y */
+                                                   bmp_width, /* dst_width */
+                                                   bmp_height, /* dst_height */
+                                                   dst_bmp,
+                                                   error))
+    {
+      cogl_object_unref (dst_bmp);
+      cogl_object_unref (atlas_tex);
+      return NULL;
+    }
 
   cogl_object_unref (dst_bmp);
 
@@ -777,31 +854,25 @@ _cogl_atlas_texture_new_from_bitmap (CoglBitmap      *bmp,
 }
 
 void
-_cogl_atlas_texture_add_reorganize_callback (GHookFunc callback,
+_cogl_atlas_texture_add_reorganize_callback (CoglContext *ctx,
+                                             GHookFunc callback,
                                              void *user_data)
 {
-  GHook *hook;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  hook = g_hook_alloc (&ctx->atlas_reorganize_callbacks);
+  GHook *hook = g_hook_alloc (&ctx->atlas_reorganize_callbacks);
   hook->func = callback;
   hook->data = user_data;
   g_hook_prepend (&ctx->atlas_reorganize_callbacks, hook);
 }
 
 void
-_cogl_atlas_texture_remove_reorganize_callback (GHookFunc callback,
+_cogl_atlas_texture_remove_reorganize_callback (CoglContext *ctx,
+                                                GHookFunc callback,
                                                 void *user_data)
 {
-  GHook *hook;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  hook = g_hook_find_func_data (&ctx->atlas_reorganize_callbacks,
-                                FALSE,
-                                callback,
-                                user_data);
+  GHook *hook = g_hook_find_func_data (&ctx->atlas_reorganize_callbacks,
+                                       FALSE,
+                                       callback,
+                                       user_data);
 
   if (hook)
     g_hook_destroy_link (&ctx->atlas_reorganize_callbacks, hook);
@@ -817,6 +888,7 @@ static const CoglTextureVtable
 cogl_atlas_texture_vtable =
   {
     FALSE, /* not primitive */
+    _cogl_atlas_texture_allocate,
     _cogl_atlas_texture_set_region,
     NULL, /* get_data */
     _cogl_atlas_texture_foreach_sub_texture_in_region,
@@ -826,14 +898,12 @@ cogl_atlas_texture_vtable =
     _cogl_atlas_texture_transform_coords_to_gl,
     _cogl_atlas_texture_transform_quad_coords_to_gl,
     _cogl_atlas_texture_get_gl_texture,
-    _cogl_atlas_texture_set_filters,
+    _cogl_atlas_texture_gl_flush_legacy_texobj_filters,
     _cogl_atlas_texture_pre_paint,
     _cogl_atlas_texture_ensure_non_quad_rendering,
-    _cogl_atlas_texture_set_wrap_mode_parameters,
+    _cogl_atlas_texture_gl_flush_legacy_texobj_wrap_modes,
     _cogl_atlas_texture_get_format,
     _cogl_atlas_texture_get_gl_format,
-    _cogl_atlas_texture_get_width,
-    _cogl_atlas_texture_get_height,
     _cogl_atlas_texture_get_type,
     NULL, /* is_foreign */
     NULL /* set_auto_mipmap */
