@@ -31,6 +31,8 @@
 #include "cogl-object-private.h"
 #include "cogl-attribute.h"
 #include "cogl-framebuffer.h"
+#include "cogl-pipeline-private.h"
+#include "cogl-boxed-value.h"
 
 typedef enum
 {
@@ -47,21 +49,32 @@ typedef struct _CoglAttributeNameState
   CoglAttributeNameID name_id;
   int name_index;
   CoglBool normalized_default;
-  int texture_unit;
+  int layer_number;
 } CoglAttributeNameState;
 
 struct _CoglAttribute
 {
   CoglObject _parent;
 
-  CoglAttributeBuffer *attribute_buffer;
   const CoglAttributeNameState *name_state;
-  size_t stride;
-  size_t offset;
-  int n_components;
-  CoglAttributeType type;
   CoglBool normalized;
 
+  CoglBool is_buffered;
+
+  union {
+    struct {
+      CoglAttributeBuffer *attribute_buffer;
+      size_t stride;
+      size_t offset;
+      int n_components;
+      CoglAttributeType type;
+    } buffered;
+    struct {
+      CoglContext *context;
+      CoglBoxedValue boxed;
+    } constant;
+  } d;
+
   int immutable_ref;
 };
 
@@ -99,8 +112,12 @@ _cogl_attribute_immutable_ref (CoglAttribute *attribute);
 void
 _cogl_attribute_immutable_unref (CoglAttribute *attribute);
 
-void
-_cogl_attribute_disable_cached_arrays (void);
+typedef struct
+{
+  int unit;
+  CoglPipelineFlushOptions options;
+  uint32_t fallback_layers;
+} CoglFlushLayerState;
 
 void
 _cogl_flush_attributes_state (CoglFramebuffer *framebuffer,
