@@ -32,6 +32,7 @@
 #include <string.h>
 
 #define component_type uint8_t
+#define component_size 8
 /* We want to specially optimise the packing when we are converting
    to/from an 8-bit type so that it won't do anything. That way for
    example if we are just doing a swizzle conversion then the inner
@@ -42,14 +43,17 @@
 #undef PACK_BYTE
 #undef UNPACK_BYTE
 #undef component_type
+#undef component_size
 
 #define component_type uint16_t
+#define component_size 16
 #define UNPACK_BYTE(b) (((b) * 65535 + 127) / 255)
 #define PACK_BYTE(b) (((b) * 255 + 32767) / 65535)
 #include "cogl-bitmap-packing.h"
 #undef PACK_BYTE
 #undef UNPACK_BYTE
 #undef component_type
+#undef component_size
 
 /* (Un)Premultiplication */
 
@@ -205,8 +209,8 @@ _cogl_premult_alpha_last_four_pixels_sse2 (uint8_t *p)
 #endif /* COGL_USE_PREMULT_SSE2 */
 
 static void
-_cogl_bitmap_premult_unpacked_span_uint8_t (uint8_t *data,
-                                           int width)
+_cogl_bitmap_premult_unpacked_span_8 (uint8_t *data,
+                                      int width)
 {
 #ifdef COGL_USE_PREMULT_SSE2
 
@@ -231,8 +235,8 @@ _cogl_bitmap_premult_unpacked_span_uint8_t (uint8_t *data,
 }
 
 static void
-_cogl_bitmap_unpremult_unpacked_span_uint8_t (uint8_t *data,
-                                             int width)
+_cogl_bitmap_unpremult_unpacked_span_8 (uint8_t *data,
+                                        int width)
 {
   int x;
 
@@ -247,8 +251,8 @@ _cogl_bitmap_unpremult_unpacked_span_uint8_t (uint8_t *data,
 }
 
 static void
-_cogl_bitmap_unpremult_unpacked_span_uint16_t (uint16_t *data,
-                                              int width)
+_cogl_bitmap_unpremult_unpacked_span_16 (uint16_t *data,
+                                         int width)
 {
   while (width-- > 0)
     {
@@ -266,8 +270,8 @@ _cogl_bitmap_unpremult_unpacked_span_uint16_t (uint16_t *data,
 }
 
 static void
-_cogl_bitmap_premult_unpacked_span_uint16_t (uint16_t *data,
-                                            int width)
+_cogl_bitmap_premult_unpacked_span_16 (uint16_t *data,
+                                       int width)
 {
   while (width-- > 0)
     {
@@ -306,6 +310,9 @@ _cogl_bitmap_needs_short_temp_buffer (CoglPixelFormat format)
      floats */
   switch (format)
     {
+    case COGL_PIXEL_FORMAT_DEPTH_16:
+    case COGL_PIXEL_FORMAT_DEPTH_32:
+    case COGL_PIXEL_FORMAT_DEPTH_24_STENCIL_8:
     case COGL_PIXEL_FORMAT_ANY:
     case COGL_PIXEL_FORMAT_YUV:
       g_assert_not_reached ();
@@ -345,7 +352,8 @@ _cogl_bitmap_needs_short_temp_buffer (CoglPixelFormat format)
 
 CoglBool
 _cogl_bitmap_convert_into_bitmap (CoglBitmap *src_bmp,
-                                  CoglBitmap *dst_bmp)
+                                  CoglBitmap *dst_bmp,
+                                  CoglError **error)
 {
   uint8_t *src_data;
   uint8_t *dst_data;
@@ -385,19 +393,20 @@ _cogl_bitmap_convert_into_bitmap (CoglBitmap *src_bmp,
       if (!_cogl_bitmap_copy_subregion (src_bmp, dst_bmp,
                                         0, 0, /* src_x / src_y */
                                         0, 0, /* dst_x / dst_y */
-                                        width, height))
+                                        width, height,
+                                        error))
         return FALSE;
 
       if (need_premult)
         {
           if ((dst_format & COGL_PREMULT_BIT))
             {
-              if (!_cogl_bitmap_premult (dst_bmp))
+              if (!_cogl_bitmap_premult (dst_bmp, error))
                 return FALSE;
             }
           else
             {
-              if (!_cogl_bitmap_unpremult (dst_bmp))
+              if (!_cogl_bitmap_unpremult (dst_bmp, error))
                 return FALSE;
             }
         }
@@ -405,12 +414,13 @@ _cogl_bitmap_convert_into_bitmap (CoglBitmap *src_bmp,
       return TRUE;
     }
 
-  src_data = _cogl_bitmap_map (src_bmp, COGL_BUFFER_ACCESS_READ, 0);
+  src_data = _cogl_bitmap_map (src_bmp, COGL_BUFFER_ACCESS_READ, 0, error);
   if (src_data == NULL)
     return FALSE;
   dst_data = _cogl_bitmap_map (dst_bmp,
                                COGL_BUFFER_ACCESS_WRITE,
-                               COGL_BUFFER_MAP_HINT_DISCARD);
+                               COGL_BUFFER_MAP_HINT_DISCARD,
+                               error);
   if (dst_data == NULL)
     {
       _cogl_bitmap_unmap (src_bmp);
@@ -430,9 +440,9 @@ _cogl_bitmap_convert_into_bitmap (CoglBitmap *src_bmp,
       dst = dst_data + y * dst_rowstride;
 
       if (use_16)
-        _cogl_unpack_uint16_t (src_format, src, tmp_row, width);
+        _cogl_unpack_16 (src_format, src, tmp_row, width);
       else
-        _cogl_unpack_uint8_t (src_format, src, tmp_row, width);
+        _cogl_unpack_8 (src_format, src, tmp_row, width);
 
       /* Handle premultiplication */
       if (need_premult)
@@ -440,23 +450,23 @@ _cogl_bitmap_convert_into_bitmap (CoglBitmap *src_bmp,
           if (dst_format & COGL_PREMULT_BIT)
             {
               if (use_16)
-                _cogl_bitmap_premult_unpacked_span_uint16_t (tmp_row, width);
+                _cogl_bitmap_premult_unpacked_span_16 (tmp_row, width);
               else
-                _cogl_bitmap_premult_unpacked_span_uint8_t (tmp_row, width);
+                _cogl_bitmap_premult_unpacked_span_8 (tmp_row, width);
             }
           else
             {
               if (use_16)
-                _cogl_bitmap_unpremult_unpacked_span_uint16_t (tmp_row, width);
+                _cogl_bitmap_unpremult_unpacked_span_16 (tmp_row, width);
               else
-                _cogl_bitmap_unpremult_unpacked_span_uint8_t (tmp_row, width);
+                _cogl_bitmap_unpremult_unpacked_span_8 (tmp_row, width);
             }
         }
 
       if (use_16)
-        _cogl_pack_uint16_t (dst_format, tmp_row, dst, width);
+        _cogl_pack_16 (dst_format, tmp_row, dst, width);
       else
-        _cogl_pack_uint8_t (dst_format, tmp_row, dst, width);
+        _cogl_pack_8 (dst_format, tmp_row, dst, width);
     }
 
   _cogl_bitmap_unmap (src_bmp);
@@ -469,7 +479,8 @@ _cogl_bitmap_convert_into_bitmap (CoglBitmap *src_bmp,
 
 CoglBitmap *
 _cogl_bitmap_convert (CoglBitmap *src_bmp,
-                      CoglPixelFormat dst_format)
+                      CoglPixelFormat dst_format,
+                      CoglError **error)
 {
   CoglBitmap *dst_bmp;
   int width, height;
@@ -481,9 +492,12 @@ _cogl_bitmap_convert (CoglBitmap *src_bmp,
 
   dst_bmp = _cogl_bitmap_new_with_malloc_buffer (ctx,
                                                  width, height,
-                                                 dst_format);
+                                                 dst_format,
+                                                 error);
+  if (!dst_bmp)
+    return NULL;
 
-  if (!_cogl_bitmap_convert_into_bitmap (src_bmp, dst_bmp))
+  if (!_cogl_bitmap_convert_into_bitmap (src_bmp, dst_bmp, error))
     {
       cogl_object_unref (dst_bmp);
       return NULL;
@@ -493,7 +507,8 @@ _cogl_bitmap_convert (CoglBitmap *src_bmp,
 }
 
 CoglBool
-_cogl_bitmap_unpremult (CoglBitmap *bmp)
+_cogl_bitmap_unpremult (CoglBitmap *bmp,
+                        CoglError **error)
 {
   uint8_t *p, *data;
   uint16_t *tmp_row;
@@ -510,12 +525,13 @@ _cogl_bitmap_unpremult (CoglBitmap *bmp)
   if ((data = _cogl_bitmap_map (bmp,
                                 COGL_BUFFER_ACCESS_READ |
                                 COGL_BUFFER_ACCESS_WRITE,
-                                0)) == NULL)
+                                0,
+                                error)) == NULL)
     return FALSE;
 
   /* If we can't directly unpremult the data inline then we'll
      allocate a temporary row and unpack the data. This assumes if we
-     can fast premult then we can also fast unpremult */
+      can fast premult then we can also fast unpremult */
   if (_cogl_bitmap_can_fast_premult (format))
     tmp_row = NULL;
   else
@@ -527,9 +543,9 @@ _cogl_bitmap_unpremult (CoglBitmap *bmp)
 
       if (tmp_row)
         {
-          _cogl_unpack_uint16_t (format, p, tmp_row, width);
-          _cogl_bitmap_unpremult_unpacked_span_uint16_t (tmp_row, width);
-          _cogl_pack_uint16_t (format, tmp_row, p, width);
+          _cogl_unpack_16 (format, p, tmp_row, width);
+          _cogl_bitmap_unpremult_unpacked_span_16 (tmp_row, width);
+          _cogl_pack_16 (format, tmp_row, p, width);
         }
       else
         {
@@ -545,7 +561,7 @@ _cogl_bitmap_unpremult (CoglBitmap *bmp)
                 }
             }
           else
-            _cogl_bitmap_unpremult_unpacked_span_uint8_t (p, width);
+            _cogl_bitmap_unpremult_unpacked_span_8 (p, width);
         }
     }
 
@@ -559,7 +575,8 @@ _cogl_bitmap_unpremult (CoglBitmap *bmp)
 }
 
 CoglBool
-_cogl_bitmap_premult (CoglBitmap *bmp)
+_cogl_bitmap_premult (CoglBitmap *bmp,
+                      CoglError **error)
 {
   uint8_t *p, *data;
   uint16_t *tmp_row;
@@ -576,7 +593,8 @@ _cogl_bitmap_premult (CoglBitmap *bmp)
   if ((data = _cogl_bitmap_map (bmp,
                                 COGL_BUFFER_ACCESS_READ |
                                 COGL_BUFFER_ACCESS_WRITE,
-                                0)) == NULL)
+                                0,
+                                error)) == NULL)
     return FALSE;
 
   /* If we can't directly premult the data inline then we'll allocate
@@ -592,9 +610,9 @@ _cogl_bitmap_premult (CoglBitmap *bmp)
 
       if (tmp_row)
         {
-          _cogl_unpack_uint16_t (format, p, tmp_row, width);
-          _cogl_bitmap_premult_unpacked_span_uint16_t (tmp_row, width);
-          _cogl_pack_uint16_t (format, tmp_row, p, width);
+          _cogl_unpack_16 (format, p, tmp_row, width);
+          _cogl_bitmap_premult_unpacked_span_16 (tmp_row, width);
+          _cogl_pack_16 (format, tmp_row, p, width);
         }
       else
         {
@@ -607,7 +625,7 @@ _cogl_bitmap_premult (CoglBitmap *bmp)
                 }
             }
           else
-            _cogl_bitmap_premult_unpacked_span_uint8_t (p, width);
+            _cogl_bitmap_premult_unpacked_span_8 (p, width);
         }
     }
 
