@@ -35,22 +35,26 @@
 #include "cogl-context.h"
 #include "cogl-gl-header.h"
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 typedef struct _CoglBufferVtable CoglBufferVtable;
 
 struct _CoglBufferVtable
 {
-  void * (* map) (CoglBuffer       *buffer,
-                  CoglBufferAccess  access,
-                  CoglBufferMapHint hints);
+  void * (* map_range) (CoglBuffer *buffer,
+                        size_t offset,
+                        size_t size,
+                        CoglBufferAccess access,
+                        CoglBufferMapHint hints,
+                        CoglError **error);
 
   void (* unmap) (CoglBuffer *buffer);
 
-  CoglBool (* set_data) (CoglBuffer   *buffer,
-                         unsigned int  offset,
-                         const void   *data,
-                         unsigned int  size);
+  CoglBool (* set_data) (CoglBuffer *buffer,
+                         unsigned int offset,
+                         const void *data,
+                         unsigned int size,
+                         CoglError **error);
 };
 
 typedef enum _CoglBufferFlags
@@ -113,24 +117,16 @@ _cogl_buffer_register_buffer_type (const CoglObjectClass *klass);
    _cogl_buffer_register_buffer_type (&_cogl_##type_name##_class))
 
 void
-_cogl_buffer_initialize (CoglBuffer          *buffer,
-                         CoglContext         *context,
-                         unsigned int         size,
-                         CoglBool             use_malloc,
+_cogl_buffer_initialize (CoglBuffer *buffer,
+                         CoglContext *context,
+                         size_t size,
                          CoglBufferBindTarget default_target,
-                         CoglBufferUsageHint  usage_hint,
+                         CoglBufferUsageHint usage_hint,
                          CoglBufferUpdateHint update_hint);
 
 void
 _cogl_buffer_fini (CoglBuffer *buffer);
 
-void *
-_cogl_buffer_bind (CoglBuffer *buffer,
-                   CoglBufferBindTarget target);
-
-void
-_cogl_buffer_unbind (CoglBuffer *buffer);
-
 CoglBufferUsageHint
 _cogl_buffer_get_usage_hint (CoglBuffer *buffer);
 
@@ -143,18 +139,35 @@ _cogl_buffer_immutable_ref (CoglBuffer *buffer);
 void
 _cogl_buffer_immutable_unref (CoglBuffer *buffer);
 
-/* This is a wrapper around cogl_buffer_map for internal use when we
-   want to map the buffer for write only to replace the entire
+CoglBool
+_cogl_buffer_set_data (CoglBuffer *buffer,
+                       size_t offset,
+                       const void *data,
+                       size_t size,
+                       CoglError **error);
+
+void *
+_cogl_buffer_map (CoglBuffer *buffer,
+                  CoglBufferAccess access,
+                  CoglBufferMapHint hints,
+                  CoglError **error);
+
+/* This is a wrapper around cogl_buffer_map_range for internal use
+   when we want to map the buffer for write only to replace the entire
    contents. If the map fails then it will fallback to writing to a
    temporary buffer. When _cogl_buffer_unmap_for_fill_or_fallback is
    called the temporary buffer will be copied into the array. Note
    that these calls share a global array so they can not be nested. */
 void *
+_cogl_buffer_map_range_for_fill_or_fallback (CoglBuffer *buffer,
+                                             size_t offset,
+                                             size_t size);
+void *
 _cogl_buffer_map_for_fill_or_fallback (CoglBuffer *buffer);
 
 void
 _cogl_buffer_unmap_for_fill_or_fallback (CoglBuffer *buffer);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_BUFFER_PRIVATE_H__ */
