@@ -108,7 +108,7 @@ cogl_color_set_from_4f (CoglColor *color,
 
 void
 cogl_color_init_from_4fv (CoglColor *color,
-                          float *color_array)
+                          const float *color_array)
 {
   _COGL_RETURN_IF_FAIL (color != NULL);
 
