@@ -50,6 +50,7 @@
 #include "cogl-gpu-info-private.h"
 #include "cogl-gl-header.h"
 #include "cogl-framebuffer-private.h"
+#include "cogl-onscreen-private.h"
 
 typedef struct
 {
@@ -190,6 +191,11 @@ struct _CoglContext
   gboolean have_last_offscreen_allocate_flags;
   CoglOffscreenAllocateFlags last_offscreen_allocate_flags;
 
+  GHashTable *swap_callback_closures;
+  int next_swap_callback_id;
+
+  CoglOnscreenEventList onscreen_events_queue;
+
   CoglGLES2Context *current_gles2_context;
   GQueue gles2_context_stack;
 
@@ -237,14 +243,6 @@ struct _CoglContext
   CoglBool current_gl_dither_enabled;
   CoglColorMask current_gl_color_mask;
 
-  /* List of types that will be considered a subclass of CoglTexture in
-     cogl_is_texture */
-  GSList           *texture_types;
-
-  /* List of types that will be considered a subclass of CoglBuffer in
-     cogl_is_buffer */
-  GSList           *buffer_types;
-
   /* Clipping */
   /* TRUE if we have a valid clipping stack flushed. In that case
      current_clip_stack will describe what the current state is. If
