@@ -58,11 +58,15 @@
 #include "cogl-pipeline-fragend-arbfp-private.h"
 #endif
 
-/* This isn't defined in the GLES headers */
+/* These aren't defined in the GLES headers */
 #ifndef GL_POINT_SPRITE
 #define GL_POINT_SPRITE 0x8861
 #endif
 
+#ifndef GL_NUM_EXTENSIONS
+#define GL_NUM_EXTENSIONS 0x821D
+#endif
+
 static void _cogl_context_free (CoglContext *context);
 
 COGL_OBJECT_DEFINE (Context, context);
@@ -175,9 +179,6 @@ cogl_context_new (CoglDisplay *display,
   context->feature_flags = 0;
   context->private_feature_flags = 0;
 
-  context->texture_types = NULL;
-  context->buffer_types = NULL;
-
   context->rectangle_state = COGL_WINSYS_RECTANGLE_STATE_UNKNOWN;
 
   memset (context->winsys_features, 0, sizeof (context->winsys_features));
@@ -308,6 +309,11 @@ cogl_context_new (CoglDisplay *display,
   context->current_draw_buffer_state_flushed = 0;
   context->current_draw_buffer_changes = COGL_FRAMEBUFFER_STATE_ALL;
 
+  context->swap_callback_closures =
+    g_hash_table_new (g_direct_hash, g_direct_equal);
+
+  COGL_TAILQ_INIT (&context->onscreen_events_queue);
+
   g_queue_init (&context->gles2_context_stack);
 
   context->journal_flush_attributes_array =
@@ -503,6 +509,9 @@ _cogl_context_free (CoglContext *context)
   if (context->blit_texture_pipeline)
     cogl_object_unref (context->blit_texture_pipeline);
 
+  if (context->swap_callback_closures)
+    g_hash_table_destroy (context->swap_callback_closures);
+
   g_warn_if_fail (context->gles2_context_stack.length == 0);
 
   if (context->journal_flush_attributes_array)
@@ -547,9 +556,6 @@ _cogl_context_free (CoglContext *context)
   _cogl_bitmask_destroy (&context->enable_custom_attributes_tmp);
   _cogl_bitmask_destroy (&context->changed_bits_tmp);
 
-  g_slist_free (context->texture_types);
-  g_slist_free (context->buffer_types);
-
   if (context->current_modelview_entry)
     cogl_matrix_entry_unref (context->current_modelview_entry);
   if (context->current_projection_entry)
@@ -649,6 +655,7 @@ _cogl_context_get_gl_extensions (CoglContext *context)
 
   /* In GL 3, querying GL_EXTENSIONS is deprecated so we have to build
    * the array using glGetStringi instead */
+#ifdef HAVE_COGL_GL
   if (context->driver == COGL_DRIVER_GL3)
     {
       int num_extensions, i;
@@ -667,6 +674,7 @@ _cogl_context_get_gl_extensions (CoglContext *context)
       ret[num_extensions] = NULL;
     }
   else
+#endif
     {
       const char *all_extensions =
         (const char *) context->glGetString (GL_EXTENSIONS);
@@ -742,4 +750,16 @@ _cogl_context_get_gl_version (CoglContext *context)
     return _cogl_config_override_gl_version;
   else
     return (const char *) context->glGetString (GL_VERSION);
+
+}
+
+int64_t
+cogl_get_clock_time (CoglContext *context)
+{
+  const CoglWinsysVtable *winsys = _cogl_context_get_winsys (context);
+
+  if (winsys->context_get_clock_time)
+    return winsys->context_get_clock_time (context);
+  else
+    return 0;
 }
