@@ -41,7 +41,7 @@
  * used since they don't affect the behaviour of Cogl they
  * simply print out verbose information */
 static const GDebugKey cogl_log_debug_keys[] = {
-  { "handle", COGL_DEBUG_HANDLE },
+  { "object", COGL_DEBUG_OBJECT },
   { "slicing", COGL_DEBUG_SLICING },
   { "atlas", COGL_DEBUG_ATLAS },
   { "blend-strings", COGL_DEBUG_BLEND_STRINGS },
@@ -56,7 +56,8 @@ static const GDebugKey cogl_log_debug_keys[] = {
   { "texture-pixmap", COGL_DEBUG_TEXTURE_PIXMAP },
   { "bitmap", COGL_DEBUG_BITMAP },
   { "clipping", COGL_DEBUG_CLIPPING },
-  { "winsys", COGL_DEBUG_WINSYS }
+  { "winsys", COGL_DEBUG_WINSYS },
+  { "performance", COGL_DEBUG_PERFORMANCE }
 };
 static const int n_cogl_log_debug_keys =
   G_N_ELEMENTS (cogl_log_debug_keys);
@@ -89,16 +90,16 @@ GHashTable *_cogl_debug_instances;
 
 static void
 _cogl_parse_debug_string_for_keys (const char *value,
-                                   gboolean enable,
+                                   CoglBool enable,
                                    const GDebugKey *keys,
                                    unsigned int nkeys)
 {
   int long_num, key_num;
 
   /* g_parse_debug_string expects the value field in GDebugKey to be a
-     mask in a guint but the flags is stored in an array of multiple
-     longs so we need to build a separate array for each possible
-     guint */
+     mask in an unsigned int but the flags are stored in an array of
+     multiple longs so we need to build a separate array for each
+     possible unsigned int */
 
   for (long_num = 0; long_num < COGL_DEBUG_N_LONGS; long_num++)
     {
@@ -147,8 +148,8 @@ _cogl_parse_debug_string_for_keys (const char *value,
 
 void
 _cogl_parse_debug_string (const char *value,
-                          gboolean enable,
-                          gboolean ignore_help)
+                          CoglBool enable,
+                          CoglBool ignore_help)
 {
   if (ignore_help && strcmp (value, "help") == 0)
     return;
@@ -181,6 +182,16 @@ _cogl_parse_debug_string (const char *value,
       OPT (IGNORED, "ignored", "verbose", "ignored", \
            N_("Enables all non-behavioural debug options"));
 #undef OPT
+
+      g_printerr ("\n"
+                  "%28s\n"
+                  " COGL_DISABLE_GL_EXTENSIONS: %s\n"
+                  "   COGL_OVERRIDE_GL_VERSION: %s\n",
+                  _("Additional environment variables:"),
+                  _("Comma-separated list of GL extensions to pretend are "
+                    "disabled"),
+                  _("Override the GL version that Cogl will assume the driver "
+                    "supports"));
       exit (1);
     }
   else
@@ -197,10 +208,10 @@ _cogl_parse_debug_string (const char *value,
 }
 
 #ifdef COGL_ENABLE_DEBUG
-static gboolean
+static CoglBool
 cogl_arg_debug_cb (const char *key,
                    const char *value,
-                   gpointer    user_data)
+                   void *user_data)
 {
   _cogl_parse_debug_string (value,
                             TRUE /* enable the flags */,
@@ -208,10 +219,10 @@ cogl_arg_debug_cb (const char *key,
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 cogl_arg_no_debug_cb (const char *key,
                       const char *value,
-                      gpointer    user_data)
+                      void *user_data)
 {
   _cogl_parse_debug_string (value,
                             FALSE, /* disable the flags */
@@ -254,11 +265,11 @@ _cogl_debug_check_environment (void)
     }
 }
 
-static gboolean
-pre_parse_hook (GOptionContext  *context,
-                GOptionGroup    *group,
-                gpointer         data,
-                GError         **error)
+static CoglBool
+pre_parse_hook (GOptionContext *context,
+                GOptionGroup *group,
+                void *data,
+                GError **error)
 {
   _cogl_init ();
 
