@@ -34,7 +34,7 @@
 #include <cogl/cogl-renderer.h>
 #include <cogl/cogl-onscreen-template.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 #ifdef COGL_HAS_EGL_PLATFORM_GDL_SUPPORT
 #include <libgdl.h>
@@ -90,7 +90,7 @@ typedef struct _CoglDisplay	      CoglDisplay;
  * When a display is first allocated via cogl_display_new() it is in a
  * mutable configuration mode. It's designed this way so we can
  * extend the apis available for configuring a display without
- * requiring huge numbers of constructor arguements.
+ * requiring huge numbers of constructor arguments.
  *
  * When you have finished configuring a display object you can
  * optionally call cogl_display_setup() to explicitly apply the
@@ -124,7 +124,7 @@ cogl_display_get_renderer (CoglDisplay *display);
 /**
  * cogl_display_setup:
  * @display: a #CoglDisplay
- * @error: return location for a #GError
+ * @error: return location for a #CoglError
  *
  * Explicitly sets up the given @display object. Use of this api is
  * optional since Cogl will internally setup the display if not done
@@ -133,7 +133,7 @@ cogl_display_get_renderer (CoglDisplay *display);
  * When a display is first allocated via cogl_display_new() it is in a
  * mutable configuration mode. This allows us to extend the apis
  * available for configuring a display without requiring huge numbers
- * of constructor arguements.
+ * of constructor arguments.
  *
  * Its possible to request a configuration that might not be
  * supportable on the current system and so this api provides a means
@@ -154,12 +154,13 @@ cogl_display_get_renderer (CoglDisplay *display);
  */
 CoglBool
 cogl_display_setup (CoglDisplay *display,
-                    GError **error);
+                    CoglError **error);
 
 #ifdef COGL_HAS_EGL_PLATFORM_GDL_SUPPORT
 /**
  * cogl_gdl_display_set_plane:
  * @display: a #CoglDisplay
+ * @plane: the GDL plane id
  *
  * Request that Cogl output to a specific GDL overlay @plane.
  *
@@ -185,7 +186,7 @@ cogl_gdl_display_set_plane (CoglDisplay *display,
 CoglBool
 cogl_is_display (void *object);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_DISPLAY_H__ */
 
