@@ -28,8 +28,6 @@
 #ifndef __COGL_FRAMEBUFFER_H
 #define __COGL_FRAMEBUFFER_H
 
-#include <glib.h>
-
 #ifdef COGL_HAS_WIN32_SUPPORT
 #include <windows.h>
 #endif /* COGL_HAS_WIN32_SUPPORT */
@@ -46,9 +44,10 @@
 #ifdef COGL_ENABLE_EXPERIMENTAL_API
 #include <cogl/cogl-quaternion.h>
 #include <cogl/cogl-euler.h>
+#include <cogl/cogl-texture.h>
 #endif
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-framebuffer
@@ -97,7 +96,7 @@ typedef struct _CoglFramebuffer CoglFramebuffer;
 /**
  * cogl_framebuffer_allocate:
  * @framebuffer: A #CoglFramebuffer
- * @error: A pointer to a #GError for returning exceptions.
+ * @error: A pointer to a #CoglError for returning exceptions.
  *
  * Explicitly allocates a configured #CoglFramebuffer allowing developers to
  * check and handle any errors that might arise from an unsupported
@@ -116,7 +115,7 @@ typedef struct _CoglFramebuffer CoglFramebuffer;
  */
 CoglBool
 cogl_framebuffer_allocate (CoglFramebuffer *framebuffer,
-                           GError **error);
+                           CoglError **error);
 
 /**
  * cogl_framebuffer_get_width:
@@ -185,7 +184,7 @@ cogl_framebuffer_set_viewport (CoglFramebuffer *framebuffer,
  * @framebuffer: A #CoglFramebuffer
  *
  * Queries the x coordinate of the viewport origin as set using cogl_framebuffer_set_viewport()
- * or the default value which is %0.
+ * or the default value which is 0.
  *
  * Return value: The x coordinate of the viewport origin.
  * Since: 1.8
@@ -199,7 +198,7 @@ cogl_framebuffer_get_viewport_x (CoglFramebuffer *framebuffer);
  * @framebuffer: A #CoglFramebuffer
  *
  * Queries the y coordinate of the viewport origin as set using cogl_framebuffer_set_viewport()
- * or the default value which is %0.
+ * or the default value which is 0.
  *
  * Return value: The y coordinate of the viewport origin.
  * Since: 1.8
@@ -239,7 +238,8 @@ cogl_framebuffer_get_viewport_height (CoglFramebuffer *framebuffer);
 /**
  * cogl_framebuffer_get_viewport4fv:
  * @framebuffer: A #CoglFramebuffer
- * @viewport: A pointer to an array of 4 floats to receive the (x, y, width, height)
+ * @viewport: (out caller-allocates) (array fixed-size=4): A pointer to an
+ *            array of 4 floats to receive the (x, y, width, height)
  *            components of the current viewport.
  *
  * Queries the x, y, width and height components of the current viewport as set
@@ -336,10 +336,10 @@ cogl_framebuffer_translate (CoglFramebuffer *framebuffer,
  * @z: Z-component of vertex to rotate around.
  *
  * Multiplies the current model-view matrix by one that rotates the
- * model around the vertex specified by @x, @y and @z. The rotation
- * follows the right-hand thumb rule so for example rotating by 10
- * degrees about the vertex (0, 0, 1) causes a small counter-clockwise
- * rotation.
+ * model around the axis-vector specified by @x, @y and @z. The
+ * rotation follows the right-hand thumb rule so for example rotating
+ * by 10 degrees about the axis-vector (0, 0, 1) causes a small
+ * counter-clockwise rotation.
  *
  * Since: 1.10
  * Stability: unstable
@@ -620,11 +620,9 @@ cogl_framebuffer_push_path_clip (CoglFramebuffer *framebuffer,
  *             bounds
  * @bounds_y1: y coordinate for the top-left corner of the primitives
  *             bounds
- * @bounds_x2: x coordinate for the top-left corner of the primitives
- *             bounds
- * @bounds_y2: x coordinate for the bottom-right corner of the
+ * @bounds_x2: x coordinate for the bottom-right corner of the
  *             primitives bounds.
- * @bounds_x1: y coordinate for the bottom-right corner of the
+ * @bounds_y2: y coordinate for the bottom-right corner of the
  *             primitives bounds.
  *
  * Sets a new clipping area using a 2D shaped described with a
@@ -656,7 +654,8 @@ cogl_framebuffer_push_primitive_clip (CoglFramebuffer *framebuffer,
  * @framebuffer: A #CoglFramebuffer pointer
  *
  * Reverts the clipping region to the state before the last call to
- * cogl_framebuffer_push_clip().
+ * cogl_framebuffer_push_scissor_clip(), cogl_framebuffer_push_rectangle_clip()
+ * cogl_framebuffer_push_path_clip(), or cogl_framebuffer_push_primitive_clip().
  *
  * Since: 1.10
  * Stability: unstable
@@ -720,6 +719,20 @@ cogl_framebuffer_get_blue_bits (CoglFramebuffer *framebuffer);
 int
 cogl_framebuffer_get_alpha_bits (CoglFramebuffer *framebuffer);
 
+/**
+ * cogl_framebuffer_get_depth_bits:
+ * @framebuffer: a pointer to a #CoglFramebuffer
+ *
+ * Retrieves the number of depth bits of @framebuffer
+ *
+ * Return value: the number of bits
+ *
+ * Since: 2.0
+ * Stability: unstable
+ */
+int
+cogl_framebuffer_get_depth_bits (CoglFramebuffer *framebuffer);
+
 /**
  * cogl_framebuffer_get_dither_enabled:
  * @framebuffer: a pointer to a #CoglFramebuffer
@@ -811,6 +824,65 @@ cogl_framebuffer_set_color_mask (CoglFramebuffer *framebuffer,
 CoglPixelFormat
 cogl_framebuffer_get_color_format (CoglFramebuffer *framebuffer);
 
+/**
+ * cogl_framebuffer_set_depth_texture_enabled:
+ * @framebuffer: A #CoglFramebuffer
+ * @enabled: TRUE or FALSE
+ *
+ * If @enabled is #TRUE, the depth buffer used when rendering to @framebuffer
+ * is available as a texture. You can retrieve the texture with
+ * cogl_framebuffer_get_depth_texture().
+ *
+ * <note>It's possible that your GPU does not support depth textures. You
+ * should check the %COGL_FEATURE_ID_DEPTH_TEXTURE feature before using this
+ * function.</note>
+ * <note>It's not valid to call this function after the framebuffer has been
+ * allocated as the creation of the depth texture is done at allocation time.
+ * </note>
+ *
+ * Since: 1.14
+ * Stability: unstable
+ */
+void
+cogl_framebuffer_set_depth_texture_enabled (CoglFramebuffer *framebuffer,
+                                            CoglBool enabled);
+
+/**
+ * cogl_framebuffer_get_depth_texture_enabled:
+ * @framebuffer: A #CoglFramebuffer
+ *
+ * Queries whether texture based depth buffer has been enabled via
+ * cogl_framebuffer_set_depth_texture_enabled().
+ *
+ * Return value: %TRUE if a depth texture has been enabled, else
+ *               %FALSE.
+ *
+ * Since: 1.14
+ * Stability: unstable
+ */
+CoglBool
+cogl_framebuffer_get_depth_texture_enabled (CoglFramebuffer *framebuffer);
+
+/**
+ * cogl_framebuffer_get_depth_texture:
+ * @framebuffer: A #CoglFramebuffer
+ *
+ * Retrieves the depth buffer of @framebuffer as a #CoglTexture. You need to
+ * call cogl_framebuffer_get_depth_texture(fb, TRUE); before using this
+ * function.
+ *
+ * <note>Calling this function implicitely allocates the framebuffer.</note>
+ * <note>The texture returned stays valid as long as the framebuffer stays
+ * valid.</note>
+ *
+ * Returns: (transfer none): the depth texture
+ *
+ * Since: 1.14
+ * Stability: unstable
+ */
+CoglTexture *
+cogl_framebuffer_get_depth_texture (CoglFramebuffer *framebuffer);
+
 /**
  * cogl_framebuffer_set_samples_per_pixel:
  * @framebuffer: A #CoglFramebuffer framebuffer
@@ -956,6 +1028,7 @@ cogl_framebuffer_resolve_samples_region (CoglFramebuffer *framebuffer,
                                          int height);
 
 /**
+ * cogl_framebuffer_get_context:
  * @framebuffer: A #CoglFramebuffer
  *
  * Can be used to query the #CoglContext a given @framebuffer was
@@ -1080,7 +1153,7 @@ cogl_framebuffer_vdraw_attributes (CoglFramebuffer *framebuffer,
                                    CoglVerticesMode mode,
                                    int first_vertex,
                                    int n_vertices,
-                                   ...) G_GNUC_NULL_TERMINATED;
+                                   ...) COGL_GNUC_NULL_TERMINATED;
 
 /**
  * cogl_framebuffer_draw_attributes:
@@ -1188,7 +1261,7 @@ cogl_framebuffer_vdraw_indexed_attributes (CoglFramebuffer *framebuffer,
                                            int first_vertex,
                                            int n_vertices,
                                            CoglIndices *indices,
-                                           ...) G_GNUC_NULL_TERMINATED;
+                                           ...) COGL_GNUC_NULL_TERMINATED;
 
 /**
  * cogl_framebuffer_draw_indexed_attributes:
@@ -1328,7 +1401,7 @@ cogl_framebuffer_draw_rectangle (CoglFramebuffer *framebuffer,
  * The given texture coordinates should always be normalized such that
  * (0, 0) corresponds to the top left and (1, 1) corresponds to the
  * bottom right. To map an entire texture across the rectangle pass
- * in @s_1=%0, @t_1=%0, @s_2=%1, @t_2=%1.
+ * in @s_1=0, @t_1=0, @s_2=1, @t_2=1.
  *
  * <note>Even if you have associated a #CoglTextureRectangle texture
  * with one of your @pipeline layers which normally implies working
@@ -1402,8 +1475,8 @@ cogl_framebuffer_draw_textured_rectangle (CoglFramebuffer *framebuffer,
  * The given texture coordinates should always be normalized such that
  * (0, 0) corresponds to the top left and (1, 1) corresponds to the
  * bottom right. To map an entire texture across the rectangle pass
- * in tex_coords[0]=%0, tex_coords[1]=%0, tex_coords[2]=%1,
- * tex_coords[3]=%1.
+ * in tex_coords[0]=0, tex_coords[1]=0, tex_coords[2]=1,
+ * tex_coords[3]=1.
  *
  * <note>Even if you have associated a #CoglTextureRectangle texture
  * which normally implies working with non-normalized texture
@@ -1465,8 +1538,8 @@ cogl_framebuffer_draw_multitextured_rectangle (CoglFramebuffer *framebuffer,
 void
 cogl_framebuffer_draw_rectangles (CoglFramebuffer *framebuffer,
                                   CoglPipeline *pipeline,
-                                  const float *verts,
-                                  unsigned int n_rects);
+                                  const float *coordinates,
+                                  unsigned int n_rectangles);
 
 /**
  * cogl_framebuffer_draw_textured_rectangles:
@@ -1512,8 +1585,8 @@ cogl_framebuffer_draw_rectangles (CoglFramebuffer *framebuffer,
  * The given texture coordinates should always be normalized such that
  * (0, 0) corresponds to the top left and (1, 1) corresponds to the
  * bottom right. To map an entire texture across the rectangle pass
- * in tex_coords[0]=%0, tex_coords[1]=%0, tex_coords[2]=%1,
- * tex_coords[3]=%1.
+ * in tex_coords[0]=0, tex_coords[1]=0, tex_coords[2]=1,
+ * tex_coords[3]=1.
  *
  * <note>Even if you have associated a #CoglTextureRectangle texture
  * which normally implies working with non-normalized texture
@@ -1722,7 +1795,7 @@ cogl_get_draw_framebuffer (void);
 /* XXX: Note these are defined outside the COGL_ENABLE_EXPERIMENTAL_API guard since
  * otherwise the glib-mkenums stuff will get upset. */
 
-GQuark
+uint32_t
 cogl_framebuffer_error_quark (void);
 
 /**
@@ -1750,7 +1823,7 @@ typedef enum { /*< prefix=COGL_FRAMEBUFFER_ERROR >*/
 CoglBool
 cogl_is_framebuffer (void *object);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_FRAMEBUFFER_H */
 
