@@ -52,13 +52,17 @@ cogl_poll_get_info (CoglContext *context,
                              poll_fds,
                              n_poll_fds,
                              timeout);
-      return;
+    }
+  else
+    {
+      /* By default we'll assume Cogl doesn't need to block on anything */
+      *poll_fds = NULL;
+      *n_poll_fds = 0;
+      *timeout = -1; /* no timeout */
     }
 
-  /* By default we'll assume Cogl doesn't need to block on anything */
-  *poll_fds = NULL;
-  *n_poll_fds = 0;
-  *timeout = -1; /* no timeout */
+  if (!COGL_TAILQ_EMPTY (&context->onscreen_events_queue))
+    *timeout = 0;
 }
 
 void
@@ -70,6 +74,9 @@ cogl_poll_dispatch (CoglContext *context,
 
   _COGL_RETURN_IF_FAIL (cogl_is_context (context));
 
+  if (!COGL_TAILQ_EMPTY (&context->onscreen_events_queue))
+    _cogl_dispatch_onscreen_events (context);
+
   winsys = _cogl_context_get_winsys (context);
 
   if (winsys->poll_dispatch)
