@@ -39,7 +39,7 @@ typedef struct _CoglPrimitive CoglPrimitive;
 #include <cogl/cogl-vertex-buffer.h> /* for CoglVerticesMode */
 #include <cogl/cogl-attribute.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-primitive
@@ -55,13 +55,12 @@ G_BEGIN_DECLS
  * @y: The y component of a position attribute
  *
  * A convenience vertex definition that can be used with
- * cogl_primitive_new_with_v2_attributes().
+ * cogl_primitive_new_p2().
  *
  * Since: 1.6
  * Stability: Unstable
  */
-typedef struct
-{
+typedef struct {
    float x, y;
 } CoglVertexP2;
 
@@ -72,13 +71,12 @@ typedef struct
  * @z: The z component of a position attribute
  *
  * A convenience vertex definition that can be used with
- * cogl_primitive_new_with_v3_attributes().
+ * cogl_primitive_new_p3().
  *
  * Since: 1.6
  * Stability: Unstable
  */
-typedef struct
-{
+typedef struct {
    float x, y, z;
 } CoglVertexP3;
 
@@ -92,13 +90,12 @@ typedef struct
  * @a: The alpha component of a color attribute
  *
  * A convenience vertex definition that can be used with
- * cogl_primitive_new_with_v2c4_attributes().
+ * cogl_primitive_new_p2c4().
  *
  * Since: 1.6
  * Stability: Unstable
  */
-typedef struct
-{
+typedef struct {
    float x, y;
    uint8_t r, g, b, a;
 } CoglVertexP2C4;
@@ -114,13 +111,12 @@ typedef struct
  * @a: The alpha component of a color attribute
  *
  * A convenience vertex definition that can be used with
- * cogl_primitive_new_with_v3c4_attributes().
+ * cogl_primitive_new_p3c4().
  *
  * Since: 1.6
  * Stability: Unstable
  */
-typedef struct
-{
+typedef struct {
    float x, y, z;
    uint8_t r, g, b, a;
 } CoglVertexP3C4;
@@ -133,13 +129,12 @@ typedef struct
  * @t: The t component of a texture coordinate attribute
  *
  * A convenience vertex definition that can be used with
- * cogl_primitive_new_with_v2t2_attributes().
+ * cogl_primitive_new_p2t2().
  *
  * Since: 1.6
  * Stability: Unstable
  */
-typedef struct
-{
+typedef struct {
    float x, y;
    float s, t;
 } CoglVertexP2T2;
@@ -153,13 +148,12 @@ typedef struct
  * @t: The t component of a texture coordinate attribute
  *
  * A convenience vertex definition that can be used with
- * cogl_primitive_new_with_v3t2_attributes().
+ * cogl_primitive_new_p3t2().
  *
  * Since: 1.6
  * Stability: Unstable
  */
-typedef struct
-{
+typedef struct {
    float x, y, z;
    float s, t;
 } CoglVertexP3T2;
@@ -177,13 +171,12 @@ typedef struct
  * @a: The alpha component of a color attribute
  *
  * A convenience vertex definition that can be used with
- * cogl_primitive_new_with_v3t2c4_attributes().
+ * cogl_primitive_new_p3t2c4().
  *
  * Since: 1.6
  * Stability: Unstable
  */
-typedef struct
-{
+typedef struct {
    float x, y;
    float s, t;
    uint8_t r, g, b, a;
@@ -202,13 +195,12 @@ typedef struct
  * @a: The alpha component of a color attribute
  *
  * A convenience vertex definition that can be used with
- * cogl_primitive_new_with_v3t2c4_attributes().
+ * cogl_primitive_new_p3t2c4().
  *
  * Since: 1.6
  * Stability: Unstable
  */
-typedef struct
-{
+typedef struct {
    float x, y, z;
    float s, t;
    uint8_t r, g, b, a;
@@ -218,14 +210,14 @@ typedef struct
  * cogl_primitive_new:
  * @mode: A #CoglVerticesMode defining how to draw the vertices
  * @n_vertices: The number of vertices to process when drawing
- * @Varargs: A %NULL terminated list of attributes
+ * @...: A %NULL terminated list of attributes
  *
  * Combines a set of #CoglAttribute<!-- -->s with a specific draw @mode
  * and defines a vertex count so a #CoglPrimitive object can be retained and
  * drawn later with no addition information required.
  *
  * The value passed as @n_vertices will simply update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
  *
@@ -273,7 +265,7 @@ cogl_primitive_new_with_attributes (CoglVerticesMode mode,
  *
  * The value passed as @n_vertices is initially used to determine how
  * much can be read from @data but it will also be used to update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
 
@@ -326,7 +318,7 @@ cogl_primitive_new_p2 (CoglContext *context,
  *
  * The value passed as @n_vertices is initially used to determine how
  * much can be read from @data but it will also be used to update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
 
@@ -381,7 +373,7 @@ cogl_primitive_new_p3 (CoglContext *context,
  *
  * The value passed as @n_vertices is initially used to determine how
  * much can be read from @data but it will also be used to update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
 
@@ -436,7 +428,7 @@ cogl_primitive_new_p2c4 (CoglContext *context,
  *
  * The value passed as @n_vertices is initially used to determine how
  * much can be read from @data but it will also be used to update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
 
@@ -491,7 +483,7 @@ cogl_primitive_new_p3c4 (CoglContext *context,
  *
  * The value passed as @n_vertices is initially used to determine how
  * much can be read from @data but it will also be used to update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
 
@@ -546,7 +538,7 @@ cogl_primitive_new_p2t2 (CoglContext *context,
  *
  * The value passed as @n_vertices is initially used to determine how
  * much can be read from @data but it will also be used to update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
 
@@ -601,7 +593,7 @@ cogl_primitive_new_p3t2 (CoglContext *context,
  *
  * The value passed as @n_vertices is initially used to determine how
  * much can be read from @data but it will also be used to update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
 
@@ -656,7 +648,7 @@ cogl_primitive_new_p2t2c4 (CoglContext *context,
  *
  * The value passed as @n_vertices is initially used to determine how
  * much can be read from @data but it will also be used to update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to read when drawing.
 
@@ -687,7 +679,6 @@ void
 cogl_primitive_set_first_vertex (CoglPrimitive *primitive,
                                  int first_vertex);
 
-#define cogl_primitive_get_n_vertices cogl_primitive_get_n_vertices_EXP
 /**
  * cogl_primitive_get_n_vertices:
  * @primitive: A #CoglPrimitive object
@@ -713,7 +704,6 @@ cogl_primitive_set_first_vertex (CoglPrimitive *primitive,
 int
 cogl_primitive_get_n_vertices (CoglPrimitive *primitive);
 
-#define cogl_primitive_set_n_vertices cogl_primitive_set_n_vertices_EXP
 /**
  * cogl_primitive_set_n_vertices:
  * @primitive: A #CoglPrimitive object
@@ -746,8 +736,8 @@ cogl_primitive_set_mode (CoglPrimitive *primitive,
 /**
  * cogl_primitive_set_attributes:
  * @primitive: A #CoglPrimitive object
- * @attributes: A %NULL terminated array of #CoglAttribute
- *              pointers
+ * @attributes: an array of #CoglAttribute pointers
+ * @n_attributes: the number of elements in @attributes
  *
  * Replaces all the attributes of the given #CoglPrimitive object.
  *
@@ -759,7 +749,6 @@ cogl_primitive_set_attributes (CoglPrimitive *primitive,
                                CoglAttribute **attributes,
                                int n_attributes);
 
-#define cogl_primitive_set_indices cogl_primitive_set_indices_EXP
 /**
  * cogl_primitive_set_indices:
  * @primitive: A #CoglPrimitive
@@ -776,14 +765,14 @@ cogl_primitive_set_attributes (CoglPrimitive *primitive,
  * shared data multiple times instead of duplicating the data.
  *
  * The value passed as @n_indices will simply update the
- * #CoglPrimitive::n_vertices property as if
+ * #CoglPrimitive <structfield>n_vertices</structfield> property as if
  * cogl_primitive_set_n_vertices() were called. This property defines
  * the number of vertices to draw or, put another way, how many
  * indices should be read from @indices when drawing.
  *
- * <note>The #CoglPrimitive::first_vertex property also affects
- * drawing with indices by defining the first entry of the indices to
- * start drawing from.</note>
+ * <note>The #CoglPrimitive <structfield>first_vertex</structfield> property
+ * also affects drawing with indices by defining the first entry of the
+ * indices to start drawing from.</note>
  *
  * Since: 1.10
  * Stability: unstable
@@ -871,7 +860,7 @@ cogl_primitive_foreach_attribute (CoglPrimitive *primitive,
                                   CoglPrimitiveAttributeCallback callback,
                                   void *user_data);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_PRIMITIVE_H__ */
 
