@@ -590,7 +590,8 @@ cogl_renderer_get_n_fragment_texture_units (CoglRenderer *renderer)
   _COGL_GET_CONTEXT (ctx, 0);
 
 #if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES2)
-  if (cogl_has_feature (ctx, COGL_FEATURE_ID_GLSL))
+  if (cogl_has_feature (ctx, COGL_FEATURE_ID_GLSL) ||
+      cogl_has_feature (ctx, COGL_FEATURE_ID_ARBFP))
     GE (ctx, glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS, &n));
 #endif
 
@@ -630,3 +631,17 @@ cogl_renderer_get_driver (CoglRenderer *renderer)
 
   return renderer->driver;
 }
+
+void
+cogl_renderer_foreach_output (CoglRenderer *renderer,
+                              CoglOutputCallback callback,
+                              void *user_data)
+{
+  GList *l;
+
+  _COGL_RETURN_IF_FAIL (renderer->connected);
+  _COGL_RETURN_IF_FAIL (callback != NULL);
+
+  for (l = renderer->outputs; l; l = l->next)
+    callback (l->data, user_data);
+}
