@@ -32,10 +32,9 @@
 #define __COGL_TEXTURE_2D_H
 
 #include "cogl-context.h"
+#include "cogl-bitmap.h"
 
-#include <glib.h>
-
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-texture-2d
@@ -56,7 +55,6 @@ G_BEGIN_DECLS
 typedef struct _CoglTexture2D CoglTexture2D;
 #define COGL_TEXTURE_2D(X) ((CoglTexture2D *)X)
 
-#define cogl_is_texture_2d cogl_is_texture_2d_EXP
 /**
  * cogl_is_texture_2d:
  * @object: A #CoglObject
@@ -70,14 +68,12 @@ typedef struct _CoglTexture2D CoglTexture2D;
 CoglBool
 cogl_is_texture_2d (void *object);
 
-#define cogl_texture_2d_new_with_size cogl_texture_2d_new_with_size_EXP
 /**
  * cogl_texture_2d_new_with_size:
  * @ctx: A #CoglContext
  * @width: Width of the texture to allocate
  * @height: Height of the texture to allocate
  * @internal_format: The format of the texture
- * @error: A #GError for exceptions
  *
  * Allocates a low-level #CoglTexture2D texture that your GPU can
  * texture from directly. This is unlike sliced textures for example
@@ -85,15 +81,18 @@ cogl_is_texture_2d (void *object);
  * textures where Cogl has to modify texture coordinates before they
  * may be used by the GPU.
  *
+ * The storage for the texture is not allocated before this function
+ * returns. You can call cogl_texture_allocate() to explicitly
+ * allocate the underlying storage or preferably let Cogl
+ * automatically allocate storage lazily when it may know more about
+ * how the texture is being used and can optimize how it is allocated.
+ *
  * <note>Many GPUs only support power of two sizes for #CoglTexture2D
  * textures. You can check support for non power of two textures by
  * checking for the %COGL_FEATURE_ID_TEXTURE_NPOT feature via
  * cogl_has_feature().</note>
  *
- * Returns: A newly allocated #CoglTexture2D, or if the size is not
- *          supported (because it is too large or a non-power-of-two
- *          size that the hardware doesn't support) it will return
- *          %NULL and set @error.
+ * Returns: A new #CoglTexture2D object with no storage yet allocated.
  *
  * Since: 2.0
  */
@@ -101,10 +100,8 @@ CoglTexture2D *
 cogl_texture_2d_new_with_size (CoglContext *ctx,
                                int width,
                                int height,
-                               CoglPixelFormat internal_format,
-                               GError **error);
+                               CoglPixelFormat internal_format);
 
-#define cogl_texture_2d_new_from_data cogl_texture_2d_new_from_data_EXP
 /**
  * cogl_texture_2d_new_from_data:
  * @ctx: A #CoglContext
@@ -123,7 +120,7 @@ cogl_texture_2d_new_with_size (CoglContext *ctx,
  *    scanlines in @data. A value of 0 will make Cogl automatically
  *    calculate @rowstride from @width and @format.
  * @data: pointer the memory region where the source buffer resides
- * @error: A #GError for exceptions
+ * @error: A #CoglError for exceptions
  *
  * Creates a new #CoglTexture2D texture based on data residing in memory.
  * These are unlike sliced textures for example which may be comprised
@@ -150,7 +147,7 @@ cogl_texture_2d_new_from_data (CoglContext *ctx,
                                CoglPixelFormat internal_format,
                                int rowstride,
                                const uint8_t *data,
-                               GError **error);
+                               CoglError **error);
 
 /**
  * cogl_texture_2d_new_from_bitmap:
@@ -163,7 +160,7 @@ cogl_texture_2d_new_from_data (CoglContext *ctx,
  *    is if you have non-premultiplied source data and are going to adjust
  *    the blend mode (see cogl_pipeline_set_blend()) or use the data for
  *    something other than straight blending.
- * @error: A #GError for exceptions
+ * @error: A #CoglError for exceptions
  *
  * Creates a new #CoglTexture2D texture based on data residing in a
  * bitmap. These are unlike sliced textures for example which may be
@@ -187,42 +184,8 @@ cogl_texture_2d_new_from_data (CoglContext *ctx,
 CoglTexture2D *
 cogl_texture_2d_new_from_bitmap (CoglBitmap *bitmap,
                                  CoglPixelFormat internal_format,
-                                 GError **error);
+                                 CoglError **error);
 
-
-#define cogl_texture_2d_new_from_foreign cogl_texture_2d_new_from_foreign_EXP
-/**
- * cogl_texture_2d_new_from_foreign:
- * @ctx: A #CoglContext
- * @gl_handle: A GL handle for a GL_TEXTURE_2D texture object
- * @width: Width of the foreign GL texture
- * @height: Height of the foreign GL texture
- * @internal_format: The format of the texture
- * @error: A #GError for exceptions
- *
- * Wraps an existing GL_TEXTURE_2D texture object as a #CoglTexture2D.
- * This can be used for integrating Cogl with software using OpenGL
- * directly.
- *
- * <note>The results are undefined for passing an invalid @gl_handle
- * or if @width or @height don't have the correct texture
- * geometry.</note>
- *
- * Returns: A newly allocated #CoglTexture2D, or if Cogl could not
- *          validate the @gl_handle in some way (perhaps because of
- *          an unsupported format) it will return %NULL and set
- *          @error.
- *
- * Since: 2.0
- */
-CoglTexture2D *
-cogl_texture_2d_new_from_foreign (CoglContext *ctx,
-                                  unsigned int gl_handle,
-                                  int width,
-                                  int height,
-                                  CoglPixelFormat format,
-                                  GError **error);
-
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_TEXTURE_2D_H */
