@@ -31,12 +31,15 @@
 #include "cogl-texture-private.h"
 #include "cogl-texture-3d-private.h"
 #include "cogl-texture-3d.h"
+#include "cogl-texture-gl-private.h"
 #include "cogl-texture-driver.h"
 #include "cogl-context-private.h"
 #include "cogl-object-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-pipeline-private.h"
 #include "cogl-pipeline-opengl-private.h"
+#include "cogl-error-private.h"
+#include "cogl-util-gl-private.h"
 
 #include <string.h>
 #include <math.h>
@@ -56,19 +59,19 @@ COGL_TEXTURE_DEFINE (Texture3D, texture_3d);
 static const CoglTextureVtable cogl_texture_3d_vtable;
 
 static void
-_cogl_texture_3d_set_wrap_mode_parameters (CoglTexture *tex,
-                                           GLenum wrap_mode_s,
-                                           GLenum wrap_mode_t,
-                                           GLenum wrap_mode_p)
+_cogl_texture_3d_gl_flush_legacy_texobj_wrap_modes (CoglTexture *tex,
+                                                    GLenum wrap_mode_s,
+                                                    GLenum wrap_mode_t,
+                                                    GLenum wrap_mode_p)
 {
   CoglTexture3D *tex_3d = COGL_TEXTURE_3D (tex);
   CoglContext *ctx = tex->context;
 
   /* Only set the wrap mode if it's different from the current value
      to avoid too many GL calls. */
-  if (tex_3d->wrap_mode_s != wrap_mode_s ||
-      tex_3d->wrap_mode_t != wrap_mode_t ||
-      tex_3d->wrap_mode_p != wrap_mode_p)
+  if (tex_3d->gl_legacy_texobj_wrap_mode_s != wrap_mode_s ||
+      tex_3d->gl_legacy_texobj_wrap_mode_t != wrap_mode_t ||
+      tex_3d->gl_legacy_texobj_wrap_mode_p != wrap_mode_p)
     {
       _cogl_bind_gl_texture_transient (GL_TEXTURE_3D,
                                        tex_3d->gl_texture,
@@ -83,16 +86,17 @@ _cogl_texture_3d_set_wrap_mode_parameters (CoglTexture *tex,
                                 GL_TEXTURE_WRAP_R,
                                 wrap_mode_p) );
 
-      tex_3d->wrap_mode_s = wrap_mode_s;
-      tex_3d->wrap_mode_t = wrap_mode_t;
-      tex_3d->wrap_mode_p = wrap_mode_p;
+      tex_3d->gl_legacy_texobj_wrap_mode_s = wrap_mode_s;
+      tex_3d->gl_legacy_texobj_wrap_mode_t = wrap_mode_t;
+      tex_3d->gl_legacy_texobj_wrap_mode_p = wrap_mode_p;
     }
 }
 
 static void
 _cogl_texture_3d_free (CoglTexture3D *tex_3d)
 {
-  _cogl_delete_gl_texture (tex_3d->gl_texture);
+  if (tex_3d->gl_texture)
+    _cogl_delete_gl_texture (tex_3d->gl_texture);
 
   /* Chain up */
   _cogl_texture_free (COGL_TEXTURE (tex_3d));
@@ -117,26 +121,26 @@ _cogl_texture_3d_create_base (CoglContext *ctx,
   CoglTexture3D *tex_3d = g_new (CoglTexture3D, 1);
   CoglTexture *tex = COGL_TEXTURE (tex_3d);
 
-  _cogl_texture_init (tex, ctx, &cogl_texture_3d_vtable);
+  _cogl_texture_init (tex, ctx, width, height, &cogl_texture_3d_vtable);
+
+  tex_3d->gl_texture = 0;
 
-  tex_3d->width = width;
-  tex_3d->height = height;
   tex_3d->depth = depth;
   tex_3d->mipmaps_dirty = TRUE;
   tex_3d->auto_mipmap = TRUE;
 
   /* We default to GL_LINEAR for both filters */
-  tex_3d->min_filter = GL_LINEAR;
-  tex_3d->mag_filter = GL_LINEAR;
+  tex_3d->gl_legacy_texobj_min_filter = GL_LINEAR;
+  tex_3d->gl_legacy_texobj_mag_filter = GL_LINEAR;
 
   /* Wrap mode not yet set */
-  tex_3d->wrap_mode_s = GL_FALSE;
-  tex_3d->wrap_mode_t = GL_FALSE;
-  tex_3d->wrap_mode_p = GL_FALSE;
+  tex_3d->gl_legacy_texobj_wrap_mode_s = GL_FALSE;
+  tex_3d->gl_legacy_texobj_wrap_mode_t = GL_FALSE;
+  tex_3d->gl_legacy_texobj_wrap_mode_p = GL_FALSE;
 
-  tex_3d->format = internal_format;
+  tex_3d->internal_format = internal_format;
 
-  return tex_3d;
+  return _cogl_texture_3d_object_new (tex_3d);
 }
 
 static CoglBool
@@ -145,7 +149,7 @@ _cogl_texture_3d_can_create (CoglContext *ctx,
                              int height,
                              int depth,
                              CoglPixelFormat internal_format,
-                             GError **error)
+                             CoglError **error)
 {
   GLenum gl_intformat;
   GLenum gl_type;
@@ -153,10 +157,10 @@ _cogl_texture_3d_can_create (CoglContext *ctx,
   /* This should only happen on GLES */
   if (!cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_3D))
     {
-      g_set_error (error,
-                   COGL_ERROR,
-                   COGL_ERROR_UNSUPPORTED,
-                   "3D textures are not supported by the GPU");
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "3D textures are not supported by the GPU");
       return FALSE;
     }
 
@@ -167,11 +171,11 @@ _cogl_texture_3d_can_create (CoglContext *ctx,
        !_cogl_util_is_pot (height) ||
        !_cogl_util_is_pot (depth)))
     {
-      g_set_error (error,
-                   COGL_ERROR,
-                   COGL_ERROR_UNSUPPORTED,
-                   "A non-power-of-two size was requested but this is not "
-                   "supported by the GPU");
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "A non-power-of-two size was requested but this is not "
+                       "supported by the GPU");
       return FALSE;
     }
 
@@ -190,10 +194,10 @@ _cogl_texture_3d_can_create (CoglContext *ctx,
                                                height,
                                                depth))
     {
-      g_set_error (error,
-                   COGL_ERROR,
-                   COGL_ERROR_UNSUPPORTED,
-                   "The requested dimensions are not supported by the GPU");
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "The requested dimensions are not supported by the GPU");
       return FALSE;
     }
 
@@ -205,42 +209,66 @@ cogl_texture_3d_new_with_size (CoglContext *ctx,
                                int width,
                                int height,
                                int depth,
-                               CoglPixelFormat internal_format,
-                               GError **error)
+                               CoglPixelFormat internal_format)
 {
-  CoglTexture3D         *tex_3d;
-  GLenum                 gl_intformat;
-  GLenum                 gl_format;
-  GLenum                 gl_type;
-
   /* Since no data, we need some internal format */
   if (internal_format == COGL_PIXEL_FORMAT_ANY)
     internal_format = COGL_PIXEL_FORMAT_RGBA_8888_PRE;
 
+  return _cogl_texture_3d_create_base (ctx,
+                                       width, height, depth,
+                                       internal_format);
+}
+
+static CoglBool
+_cogl_texture_3d_allocate (CoglTexture *tex,
+                           CoglError **error)
+{
+  CoglContext *ctx = tex->context;
+  CoglTexture3D *tex_3d = COGL_TEXTURE_3D (tex);
+  GLenum gl_intformat;
+  GLenum gl_format;
+  GLenum gl_type;
+  GLenum gl_error;
+  GLenum gl_texture;
+
   if (!_cogl_texture_3d_can_create (ctx,
-                                    width, height, depth,
-                                    internal_format,
+                                    tex->width,
+                                    tex->height,
+                                    tex_3d->depth,
+                                    tex_3d->internal_format,
                                     error))
-    return NULL;
+    return FALSE;
 
-  internal_format = ctx->driver_vtable->pixel_format_to_gl (ctx,
-                                                            internal_format,
-                                                            &gl_intformat,
-                                                            &gl_format,
-                                                            &gl_type);
+  ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                          tex_3d->internal_format,
+                                          &gl_intformat,
+                                          &gl_format,
+                                          &gl_type);
 
-  tex_3d = _cogl_texture_3d_create_base (ctx,
-                                         width, height, depth,
-                                         internal_format);
-
-  ctx->texture_driver->gen (ctx, GL_TEXTURE_3D, 1, &tex_3d->gl_texture);
+  gl_texture =
+    ctx->texture_driver->gen (ctx, GL_TEXTURE_3D, tex_3d->internal_format);
   _cogl_bind_gl_texture_transient (GL_TEXTURE_3D,
-                                   tex_3d->gl_texture,
+                                   gl_texture,
                                    FALSE);
-  GE( ctx, glTexImage3D (GL_TEXTURE_3D, 0, gl_intformat,
-                         width, height, depth, 0, gl_format, gl_type, NULL) );
+  /* Clear any GL errors */
+  while ((gl_error = ctx->glGetError ()) != GL_NO_ERROR)
+    ;
 
-  return _cogl_texture_3d_object_new (tex_3d);
+  ctx->glTexImage3D (GL_TEXTURE_3D, 0, gl_intformat,
+                     tex->width, tex->height, tex_3d->depth,
+                     0, gl_format, gl_type, NULL);
+
+  if (_cogl_gl_util_catch_out_of_memory (ctx, error))
+    {
+      GE( ctx, glDeleteTextures (1, &gl_texture) );
+      return FALSE;
+    }
+
+  tex_3d->gl_texture = gl_texture;
+  tex_3d->gl_format = gl_intformat;
+
+  return TRUE;
 }
 
 CoglTexture3D *
@@ -248,7 +276,7 @@ cogl_texture_3d_new_from_bitmap (CoglBitmap *bmp,
                                  unsigned int height,
                                  unsigned int depth,
                                  CoglPixelFormat internal_format,
-                                 GError **error)
+                                 CoglError **error)
 {
   CoglTexture3D *tex_3d;
   CoglBitmap *dst_bmp;
@@ -257,7 +285,6 @@ cogl_texture_3d_new_from_bitmap (CoglBitmap *bmp,
   GLenum gl_intformat;
   GLenum gl_format;
   GLenum gl_type;
-  uint8_t *data;
   CoglContext *ctx;
 
   ctx = _cogl_bitmap_get_context (bmp);
@@ -279,14 +306,10 @@ cogl_texture_3d_new_from_bitmap (CoglBitmap *bmp,
                                               &internal_format,
                                               &gl_intformat,
                                               &gl_format,
-                                              &gl_type);
-
+                                              &gl_type,
+                                              error);
   if (dst_bmp == NULL)
-    {
-      g_set_error (error, COGL_BITMAP_ERROR, COGL_BITMAP_ERROR_FAILED,
-                   "Bitmap conversion failed");
-      return NULL;
-    }
+    return NULL;
 
   tex_3d = _cogl_texture_3d_create_base (ctx,
                                          bmp_width, height, depth,
@@ -294,37 +317,61 @@ cogl_texture_3d_new_from_bitmap (CoglBitmap *bmp,
 
   /* Keep a copy of the first pixel so that if glGenerateMipmap isn't
      supported we can fallback to using GL_GENERATE_MIPMAP */
-  if (!cogl_has_feature (ctx, COGL_FEATURE_ID_OFFSCREEN) &&
-      (data = _cogl_bitmap_map (dst_bmp,
-                                COGL_BUFFER_ACCESS_READ, 0)))
+  if (!cogl_has_feature (ctx, COGL_FEATURE_ID_OFFSCREEN))
     {
+      CoglError *ignore = NULL;
+      uint8_t *data = _cogl_bitmap_map (dst_bmp,
+                                        COGL_BUFFER_ACCESS_READ, 0,
+                                        &ignore);
+
       CoglPixelFormat format = cogl_bitmap_get_format (dst_bmp);
+
       tex_3d->first_pixel.gl_format = gl_format;
       tex_3d->first_pixel.gl_type = gl_type;
-      memcpy (tex_3d->first_pixel.data, data,
-              _cogl_pixel_format_get_bytes_per_pixel (format));
 
-      _cogl_bitmap_unmap (dst_bmp);
+      if (data)
+        {
+          memcpy (tex_3d->first_pixel.data, data,
+                  _cogl_pixel_format_get_bytes_per_pixel (format));
+          _cogl_bitmap_unmap (dst_bmp);
+        }
+      else
+        {
+          g_warning ("Failed to read first pixel of bitmap for "
+                     "glGenerateMipmap fallback");
+          cogl_error_free (ignore);
+          memset (tex_3d->first_pixel.data, 0,
+                  _cogl_pixel_format_get_bytes_per_pixel (format));
+        }
     }
 
-  ctx->texture_driver->gen (ctx, GL_TEXTURE_3D, 1, &tex_3d->gl_texture);
+  tex_3d->gl_texture =
+    ctx->texture_driver->gen (ctx, GL_TEXTURE_3D, internal_format);
 
-  ctx->texture_driver->upload_to_gl_3d (ctx,
-                                        GL_TEXTURE_3D,
-                                        tex_3d->gl_texture,
-                                        FALSE, /* is_foreign */
-                                        height,
-                                        depth,
-                                        dst_bmp,
-                                        gl_intformat,
-                                        gl_format,
-                                        gl_type);
+  if (!ctx->texture_driver->upload_to_gl_3d (ctx,
+                                             GL_TEXTURE_3D,
+                                             tex_3d->gl_texture,
+                                             FALSE, /* is_foreign */
+                                             height,
+                                             depth,
+                                             dst_bmp,
+                                             gl_intformat,
+                                             gl_format,
+                                             gl_type,
+                                             error))
+    {
+      cogl_object_unref (dst_bmp);
+      cogl_object_unref (tex_3d);
+      return NULL;
+    }
 
   tex_3d->gl_format = gl_intformat;
 
   cogl_object_unref (dst_bmp);
 
-  return _cogl_texture_3d_object_new (tex_3d);
+  _cogl_texture_set_allocated (COGL_TEXTURE (tex_3d), TRUE);
+
+  return tex_3d;
 }
 
 CoglTexture3D *
@@ -337,13 +384,13 @@ cogl_texture_3d_new_from_data (CoglContext *context,
                                int rowstride,
                                int image_stride,
                                const uint8_t *data,
-                               GError **error)
+                               CoglError **error)
 {
   CoglBitmap *bitmap;
   CoglTexture3D *ret;
 
   /* These are considered a programmer errors so we won't set a
-     GError. It would be nice if this was a _COGL_RETURN_IF_FAIL but the
+     CoglError. It would be nice if this was a _COGL_RETURN_IF_FAIL but the
      rest of Cogl isn't using that */
   if (format == COGL_PIXEL_FORMAT_ANY)
     return NULL;
@@ -374,11 +421,15 @@ cogl_texture_3d_new_from_data (CoglContext *context,
       bitmap = _cogl_bitmap_new_with_malloc_buffer (context,
                                                     width,
                                                     depth * height,
-                                                    format);
+                                                    format,
+                                                    error);
+      if (!bitmap)
+        return NULL;
 
       bmp_data = _cogl_bitmap_map (bitmap,
                                    COGL_BUFFER_ACCESS_WRITE,
-                                   COGL_BUFFER_MAP_HINT_DISCARD);
+                                   COGL_BUFFER_MAP_HINT_DISCARD,
+                                   error);
 
       if (bmp_data == NULL)
         {
@@ -479,20 +530,20 @@ _cogl_texture_3d_get_gl_texture (CoglTexture *tex,
 }
 
 static void
-_cogl_texture_3d_set_filters (CoglTexture *tex,
-                              GLenum       min_filter,
-                              GLenum       mag_filter)
+_cogl_texture_3d_gl_flush_legacy_texobj_filters (CoglTexture *tex,
+                                                 GLenum min_filter,
+                                                 GLenum mag_filter)
 {
   CoglTexture3D *tex_3d = COGL_TEXTURE_3D (tex);
   CoglContext *ctx = tex->context;
 
-  if (min_filter == tex_3d->min_filter
-      && mag_filter == tex_3d->mag_filter)
+  if (min_filter == tex_3d->gl_legacy_texobj_min_filter
+      && mag_filter == tex_3d->gl_legacy_texobj_mag_filter)
     return;
 
   /* Store new values */
-  tex_3d->min_filter = min_filter;
-  tex_3d->mag_filter = mag_filter;
+  tex_3d->gl_legacy_texobj_min_filter = min_filter;
+  tex_3d->gl_legacy_texobj_mag_filter = mag_filter;
 
   /* Apply new filters to the texture */
   _cogl_bind_gl_texture_transient (GL_TEXTURE_3D,
@@ -512,17 +563,18 @@ _cogl_texture_3d_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
   if ((flags & COGL_TEXTURE_NEEDS_MIPMAP) &&
       tex_3d->auto_mipmap && tex_3d->mipmaps_dirty)
     {
-      _cogl_bind_gl_texture_transient (GL_TEXTURE_3D,
-                                       tex_3d->gl_texture,
-                                       FALSE);
       /* glGenerateMipmap is defined in the FBO extension. If it's not
          available we'll fallback to temporarily enabling
          GL_GENERATE_MIPMAP and reuploading the first pixel */
       if (cogl_has_feature (ctx, COGL_FEATURE_ID_OFFSCREEN))
-        ctx->texture_driver->gl_generate_mipmaps (ctx, GL_TEXTURE_3D);
+        _cogl_texture_gl_generate_mipmaps (tex);
 #if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES)
       else if (ctx->driver != COGL_DRIVER_GLES2)
         {
+          _cogl_bind_gl_texture_transient (GL_TEXTURE_3D,
+                                           tex_3d->gl_texture,
+                                           FALSE);
+
           GE( ctx, glTexParameteri (GL_TEXTURE_3D,
                                     GL_GENERATE_MIPMAP,
                                     GL_TRUE) );
@@ -554,24 +606,32 @@ _cogl_texture_3d_ensure_non_quad_rendering (CoglTexture *tex)
 }
 
 static CoglBool
-_cogl_texture_3d_set_region (CoglTexture    *tex,
-                             int             src_x,
-                             int             src_y,
-                             int             dst_x,
-                             int             dst_y,
-                             unsigned int    dst_width,
-                             unsigned int    dst_height,
-                             CoglBitmap     *bmp)
+_cogl_texture_3d_set_region (CoglTexture *tex,
+                             int src_x,
+                             int src_y,
+                             int dst_x,
+                             int dst_y,
+                             int dst_width,
+                             int dst_height,
+                             int level,
+                             CoglBitmap *bmp,
+                             CoglError **error)
 {
   /* This function doesn't really make sense for 3D textures because
      it can't specify which image to upload to */
+  _cogl_set_error (error,
+                   COGL_SYSTEM_ERROR,
+                   COGL_SYSTEM_ERROR_UNSUPPORTED,
+                   "Setting a 2D region on a 3D texture isn't "
+                   "currently supported");
+
   return FALSE;
 }
 
 static int
 _cogl_texture_3d_get_data (CoglTexture *tex,
                            CoglPixelFormat format,
-                           unsigned int rowstride,
+                           int rowstride,
                            uint8_t *data)
 {
   /* FIXME: we could probably implement this by assuming the data is
@@ -585,7 +645,7 @@ _cogl_texture_3d_get_data (CoglTexture *tex,
 static CoglPixelFormat
 _cogl_texture_3d_get_format (CoglTexture *tex)
 {
-  return COGL_TEXTURE_3D (tex)->format;
+  return COGL_TEXTURE_3D (tex)->internal_format;
 }
 
 static GLenum
@@ -594,18 +654,6 @@ _cogl_texture_3d_get_gl_format (CoglTexture *tex)
   return COGL_TEXTURE_3D (tex)->gl_format;
 }
 
-static int
-_cogl_texture_3d_get_width (CoglTexture *tex)
-{
-  return COGL_TEXTURE_3D (tex)->width;
-}
-
-static int
-_cogl_texture_3d_get_height (CoglTexture *tex)
-{
-  return COGL_TEXTURE_3D (tex)->height;
-}
-
 static CoglTextureType
 _cogl_texture_3d_get_type (CoglTexture *tex)
 {
@@ -616,6 +664,7 @@ static const CoglTextureVtable
 cogl_texture_3d_vtable =
   {
     TRUE, /* primitive */
+    _cogl_texture_3d_allocate,
     _cogl_texture_3d_set_region,
     _cogl_texture_3d_get_data,
     NULL, /* foreach_sub_texture_in_region */
@@ -625,14 +674,12 @@ cogl_texture_3d_vtable =
     _cogl_texture_3d_transform_coords_to_gl,
     _cogl_texture_3d_transform_quad_coords_to_gl,
     _cogl_texture_3d_get_gl_texture,
-    _cogl_texture_3d_set_filters,
+    _cogl_texture_3d_gl_flush_legacy_texobj_filters,
     _cogl_texture_3d_pre_paint,
     _cogl_texture_3d_ensure_non_quad_rendering,
-    _cogl_texture_3d_set_wrap_mode_parameters,
+    _cogl_texture_3d_gl_flush_legacy_texobj_wrap_modes,
     _cogl_texture_3d_get_format,
     _cogl_texture_3d_get_gl_format,
-    _cogl_texture_3d_get_width,
-    _cogl_texture_3d_get_height,
     _cogl_texture_3d_get_type,
     NULL, /* is_foreign */
     _cogl_texture_3d_set_auto_mipmap
