@@ -29,7 +29,7 @@
 
 #include "cogl-context.h"
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-texture-rectangle
@@ -63,7 +63,6 @@ G_BEGIN_DECLS
 typedef struct _CoglTextureRectangle CoglTextureRectangle;
 #define COGL_TEXTURE_RECTANGLE(X) ((CoglTextureRectangle *)X)
 
-#define cogl_is_texture_rectangle cogl_is_texture_rectangle_EXP
 /**
  * cogl_is_texture_rectangle:
  * @object: A #CoglObject
@@ -77,21 +76,25 @@ typedef struct _CoglTextureRectangle CoglTextureRectangle;
 CoglBool
 cogl_is_texture_rectangle (void *object);
 
-#define cogl_texture_rectangle_new_with_size \
-  cogl_texture_rectangle_new_with_size_EXP
 /**
  * cogl_texture_rectangle_new_with_size:
- * @context: A #CoglContext pointer
+ * @ctx: A #CoglContext pointer
  * @width: The texture width to allocate
  * @height: The texture height to allocate
  * @internal_format: The desired internal texture format
- * @error: An optional GError pointer for reporting exceptions
+ * @error: An optional CoglError pointer for reporting exceptions
  *
  * Allocates a new #CoglTextureRectangle texture with a given @width, @height
  * and @internal_format. This texture is a low-level texture that
  * the GPU can sample from directly unlike high-level textures such
  * as #CoglTexture2DSliced and #CoglAtlasTexture.
  *
+ * <note>Unlike for #CoglTexture2D textures, coordinates for
+ * #CoglTextureRectangle textures should not be normalized. So instead
+ * of using the coordinate (1, 1) to sample the bottom right corner of
+ * a rectangle texture you would use (@width, @height) where @width
+ * and @height are the width and height of the texture.</note>
+ *
  * <note>If you want to sample from a rectangle texture from GLSL you
  * should use the sampler2DRect sampler type.</note>
  *
@@ -99,7 +102,7 @@ cogl_is_texture_rectangle (void *object);
  * first check for the %COGL_FEATURE_ID_TEXTURE_RECTANGLE feature
  * using cogl_has_feature().</note>
  *
- * Returns: A pointer to a newly allocated #CoglRectangle texture
+ * Returns: A pointer to a newly allocated #CoglTextureRectangle texture
  *          or if the size was too large or there wasn't enough memory
  *          %NULL is returned and @error set.
  *
@@ -111,14 +114,14 @@ cogl_texture_rectangle_new_with_size (CoglContext *ctx,
                                       int width,
                                       int height,
                                       CoglPixelFormat internal_format,
-                                      GError **error);
+                                      CoglError **error);
 
 /**
  * cogl_texture_rectangle_new_from_bitmap:
  * @bitmap: A #CoglBitmap
  * @internal_format: the #CoglPixelFormat to use for the GPU storage of the
  *    texture
- * @error: A return location for a GError or %NULL
+ * @error: A return location for a CoglError or %NULL
  *
  * Allocates a new #CoglTextureRectangle texture which will be
  * initialized with the pixel data from @bitmap. Internally the data
@@ -127,6 +130,12 @@ cogl_texture_rectangle_new_with_size (CoglContext *ctx,
  * directly unlike high-level textures such as #CoglTexture2DSliced
  * and #CoglAtlasTexture.
  *
+ * <note>Unlike for #CoglTexture2D textures, coordinates for
+ * #CoglTextureRectangle textures should not be normalized. So instead
+ * of using the coordinate (1, 1) to sample the bottom right corner of
+ * a rectangle texture you would use (@width, @height) where @width
+ * and @height are the width and height of the texture.</note>
+ *
  * <note>If you want to sample from a rectangle texture from GLSL you
  * should use the sampler2DRect sampler type.</note>
  *
@@ -134,7 +143,7 @@ cogl_texture_rectangle_new_with_size (CoglContext *ctx,
  * first check for the %COGL_FEATURE_ID_TEXTURE_RECTANGLE feature
  * using cogl_has_feature().</note>
  *
- * Returns: A pointer to a newly allocated #CoglRectangle texture
+ * Returns: A pointer to a newly allocated #CoglTextureRectangle texture
  *          or if the size was too large or there wasn't enough memory
  *          %NULL is returned and @error set.
  * Since: 2.0
@@ -143,8 +152,53 @@ cogl_texture_rectangle_new_with_size (CoglContext *ctx,
 CoglTextureRectangle *
 cogl_texture_rectangle_new_from_bitmap (CoglBitmap *bitmap,
                                         CoglPixelFormat internal_format,
-                                        GError **error);
+                                        CoglError **error);
 
-G_END_DECLS
+/**
+ * cogl_texture_rectangle_new_from_foreign:
+ * @ctx: A #CoglContext
+ * @gl_handle: A GL handle for a GL_TEXTURE_RECTANGLE texture object
+ * @width: Width of the foreign GL texture
+ * @height: Height of the foreign GL texture
+ * @internal_format: The format of the texture
+ * @error: A #CoglError for exceptions
+ *
+ * Wraps an existing GL_TEXTURE_RECTANGLE texture object as a
+ * #CoglTextureRectangle.  This can be used for integrating Cogl with
+ * software using OpenGL directly.
+ *
+ * <note>Unlike for #CoglTexture2D textures, coordinates for
+ * #CoglTextureRectangle textures should not be normalized. So instead
+ * of using the coordinate (1, 1) to sample the bottom right corner of
+ * a rectangle texture you would use (@width, @height) where @width
+ * and @height are the width and height of the texture.</note>
+ *
+ * <note>The results are undefined for passing an invalid @gl_handle
+ * or if @width or @height don't have the correct texture
+ * geometry.</note>
+ *
+ * <note>If you want to sample from a rectangle texture from GLSL you
+ * should use the sampler2DRect sampler type.</note>
+ *
+ * <note>Applications wanting to use #CoglTextureRectangle should
+ * first check for the %COGL_FEATURE_ID_TEXTURE_RECTANGLE feature
+ * using cogl_has_feature().</note>
+
+ * Returns: A newly allocated #CoglTextureRectangle, or if Cogl could
+ *          not validate the @gl_handle in some way (perhaps because
+ *          of an unsupported format) it will return %NULL and set
+ *          @error.
+ *
+
+ */
+CoglTextureRectangle *
+cogl_texture_rectangle_new_from_foreign (CoglContext *ctx,
+                                         unsigned int gl_handle,
+                                         int width,
+                                         int height,
+                                         CoglPixelFormat format,
+                                         CoglError **error);
+
+COGL_END_DECLS
 
 #endif /* __COGL_TEXURE_RECTANGLE_H */
