@@ -61,6 +61,11 @@
 #include <stdlib.h>
 #include <math.h>
 
+/* This isn't defined in the GLES headers */
+#ifndef GL_RED
+#define GL_RED 0x1903
+#endif
+
 uint32_t
 cogl_texture_error_quark (void)
 {
@@ -73,12 +78,12 @@ cogl_texture_error_quark (void)
  * abstract class manually.
  */
 
+static GSList *_cogl_texture_types;
+
 void
 _cogl_texture_register_texture_type (const CoglObjectClass *klass)
 {
-  _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
-
-  ctxt->texture_types = g_slist_prepend (ctxt->texture_types, (void *) klass);
+  _cogl_texture_types = g_slist_prepend (_cogl_texture_types, (void *) klass);
 }
 
 CoglBool
@@ -87,12 +92,10 @@ cogl_is_texture (void *object)
   CoglObject *obj = (CoglObject *)object;
   GSList *l;
 
-  _COGL_GET_CONTEXT (ctxt, FALSE);
-
   if (object == NULL)
     return FALSE;
 
-  for (l = ctxt->texture_types; l; l = l->next)
+  for (l = _cogl_texture_types; l; l = l->next)
     if (l->data == obj->klass)
       return TRUE;
 
