@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2008,2009,2010 Intel Corporation.
+ * Copyright (C) 2008,2009,2010,2013 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -207,6 +207,52 @@ has_replace_hook (CoglPipelineLayer *layer,
   return FALSE;
 }
 
+static CoglBool
+add_layer_declaration_cb (CoglPipelineLayer *layer,
+                          void *user_data)
+{
+  CoglPipelineShaderState *shader_state = user_data;
+  CoglTextureType texture_type =
+    _cogl_pipeline_layer_get_texture_type (layer);
+  const char *target_string;
+
+  _cogl_gl_util_get_texture_target_string (texture_type, &target_string, NULL);
+
+  g_string_append_printf (shader_state->header,
+                          "uniform sampler%s cogl_sampler%i;\n",
+                          target_string,
+                          layer->index);
+
+  return TRUE;
+}
+
+static void
+add_layer_declarations (CoglPipeline *pipeline,
+                        CoglPipelineShaderState *shader_state)
+{
+  /* We always emit sampler uniforms in case there will be custom
+   * layer snippets that want to sample arbitrary layers. */
+
+  _cogl_pipeline_foreach_layer_internal (pipeline,
+                                         add_layer_declaration_cb,
+                                         shader_state);
+}
+
+static void
+add_global_declarations (CoglPipeline *pipeline,
+                         CoglPipelineShaderState *shader_state)
+{
+  CoglSnippetHook hook = COGL_SNIPPET_HOOK_FRAGMENT_GLOBALS;
+  CoglPipelineSnippetList *snippets = get_fragment_snippets (pipeline);
+
+  /* Add the global data hooks. All of the code in these snippets is
+   * always added and only the declarations data is used */
+
+  _cogl_pipeline_snippet_generate_declarations (shader_state->header,
+                                                hook,
+                                                snippets);
+}
+
 static void
 _cogl_pipeline_fragend_glsl_start (CoglPipeline *pipeline,
                                    int n_layers,
@@ -315,6 +361,9 @@ _cogl_pipeline_fragend_glsl_start (CoglPipeline *pipeline,
   shader_state->source = ctx->codegen_source_buffer;
   COGL_LIST_INIT (&shader_state->layers);
 
+  add_layer_declarations (pipeline, shader_state);
+  add_global_declarations (pipeline, shader_state);
+
   g_string_append (shader_state->source,
                    "void\n"
                    "cogl_generated_source ()\n"
@@ -338,44 +387,6 @@ add_constant_lookup (CoglPipelineShaderState *shader_state,
                           layer->index, swizzle);
 }
 
-static void
-get_texture_target_string (CoglTextureType texture_type,
-                           const char **target_string_out,
-                           const char **swizzle_out)
-{
-  const char *target_string, *tex_coord_swizzle;
-
-  switch (texture_type)
-    {
-#if 0 /* TODO */
-    case COGL_TEXTURE_TYPE_1D:
-      target_string = "1D";
-      tex_coord_swizzle = "s";
-      break;
-#endif
-
-    case COGL_TEXTURE_TYPE_2D:
-      target_string = "2D";
-      tex_coord_swizzle = "st";
-      break;
-
-    case COGL_TEXTURE_TYPE_3D:
-      target_string = "3D";
-      tex_coord_swizzle = "stp";
-      break;
-
-    case COGL_TEXTURE_TYPE_RECTANGLE:
-      target_string = "2DRect";
-      tex_coord_swizzle = "st";
-      break;
-    }
-
-  if (target_string_out)
-    *target_string_out = target_string;
-  if (swizzle_out)
-    *swizzle_out = tex_coord_swizzle;
-}
-
 static void
 ensure_texture_lookup_generated (CoglPipelineShaderState *shader_state,
                                  CoglPipeline *pipeline,
@@ -393,9 +404,9 @@ ensure_texture_lookup_generated (CoglPipelineShaderState *shader_state,
 
   texture_type =
     _cogl_pipeline_layer_get_texture_type (layer);
-  get_texture_target_string (texture_type,
-                             &target_string,
-                             &tex_coord_swizzle);
+  _cogl_gl_util_get_texture_target_string (texture_type,
+                                           &target_string,
+                                           &tex_coord_swizzle);
 
   shader_state->unit_state[unit_index].sampled = TRUE;
 
@@ -992,24 +1003,6 @@ _cogl_pipeline_fragend_glsl_end (CoglPipeline *pipeline,
           CoglPipelineLayer *last_layer;
           LayerData *layer_data, *tmp;
 
-          /* We always emit sampler uniforms in case there will be custom
-           * layer snippets that want to sample arbitrary layers. */
-
-          COGL_LIST_FOREACH (layer_data, &shader_state->layers, list_node)
-            {
-              CoglPipelineLayer *layer = layer_data->layer;
-              CoglTextureType texture_type =
-                _cogl_pipeline_layer_get_texture_type (layer);
-              const char *target_string;
-
-              get_texture_target_string (texture_type, &target_string, NULL);
-
-              g_string_append_printf (shader_state->header,
-                                      "uniform sampler%s cogl_sampler%i;\n",
-                                      target_string,
-                                      layer->index);
-            }
-
           last_layer = COGL_LIST_FIRST (&shader_state->layers)->layer;
 
           ensure_layer_generated (pipeline, last_layer->index);
