@@ -37,6 +37,8 @@
 #include "cogl-context-private.h"
 #include "cogl-onscreen-private.h"
 #include "cogl-winsys-sdl-private.h"
+#include "cogl-error-private.h"
+#include "cogl-sdl.h"
 
 typedef struct _CoglContextSdl2
 {
@@ -52,13 +54,19 @@ typedef struct _CoglDisplaySdl2
 {
   SDL_Window *dummy_window;
   SDL_GLContext *context;
+  CoglBool pending_resize_notify;
 } CoglDisplaySdl2;
 
 typedef struct _CoglOnscreenSdl2
 {
   SDL_Window *window;
+  CoglBool pending_resize_notify;
 } CoglOnscreenSdl2;
 
+/* The key used to store a pointer to the CoglOnscreen in an
+ * SDL_Window */
+#define COGL_SDL_WINDOW_DATA_KEY "cogl-onscreen"
+
 static CoglFuncPtr
 _cogl_winsys_renderer_get_proc_address (CoglRenderer *renderer,
                                         const char *name,
@@ -86,14 +94,14 @@ _cogl_winsys_renderer_disconnect (CoglRenderer *renderer)
 
 static CoglBool
 _cogl_winsys_renderer_connect (CoglRenderer *renderer,
-                               GError **error)
+                               CoglError **error)
 {
   if (SDL_VideoInit (NULL) < 0)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
-                   COGL_WINSYS_ERROR_INIT,
-                   "SDL_Init failed: %s",
-                   SDL_GetError ());
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
+                       COGL_WINSYS_ERROR_INIT,
+                       "SDL_Init failed: %s",
+                       SDL_GetError ());
       return FALSE;
     }
 
@@ -140,7 +148,7 @@ set_gl_attribs_from_framebuffer_config (CoglFramebufferConfig *config)
 
 static CoglBool
 _cogl_winsys_display_setup (CoglDisplay *display,
-                            GError **error)
+                            CoglError **error)
 {
   CoglDisplaySdl2 *sdl_display;
   const char * (* get_string_func) (GLenum name);
@@ -157,6 +165,14 @@ _cogl_winsys_display_setup (CoglDisplay *display,
     SDL_GL_SetAttribute (SDL_GL_CONTEXT_MAJOR_VERSION, 1);
   else if (display->renderer->driver == COGL_DRIVER_GLES2)
     SDL_GL_SetAttribute (SDL_GL_CONTEXT_MAJOR_VERSION, 2);
+  else if (display->renderer->driver == COGL_DRIVER_GL3)
+    {
+      SDL_GL_SetAttribute (SDL_GL_CONTEXT_MAJOR_VERSION, 3);
+      SDL_GL_SetAttribute (SDL_GL_CONTEXT_PROFILE_MASK,
+                           SDL_GL_CONTEXT_PROFILE_CORE);
+      SDL_GL_SetAttribute (SDL_GL_CONTEXT_FLAGS,
+                           SDL_GL_CONTEXT_FORWARD_COMPATIBLE_FLAG);
+    }
 
   /* Create a dummy 1x1 window that never gets display so that we can
    * create a GL context */
@@ -167,10 +183,10 @@ _cogl_winsys_display_setup (CoglDisplay *display,
                                                 SDL_WINDOW_HIDDEN);
   if (sdl_display->dummy_window == NULL)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
-                   COGL_WINSYS_ERROR_INIT,
-                   "SDL_CreateWindow failed: %s",
-                   SDL_GetError ());
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
+                       COGL_WINSYS_ERROR_INIT,
+                       "SDL_CreateWindow failed: %s",
+                       SDL_GetError ());
       goto error;
     }
 
@@ -178,10 +194,10 @@ _cogl_winsys_display_setup (CoglDisplay *display,
 
   if (sdl_display->context == NULL)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
-                   COGL_WINSYS_ERROR_INIT,
-                   "SDL_GL_CreateContext failed: %s",
-                   SDL_GetError ());
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
+                       COGL_WINSYS_ERROR_INIT,
+                       "SDL_GL_CreateContext failed: %s",
+                       SDL_GetError ());
       goto error;
     }
 
@@ -195,13 +211,24 @@ _cogl_winsys_display_setup (CoglDisplay *display,
   switch (display->renderer->driver)
     {
     case COGL_DRIVER_GL:
+    case COGL_DRIVER_GL3:
       /* The first character of the version string will be a digit if
        * it's normal GL */
       if (!g_ascii_isdigit (gl_version[0]))
         {
-          g_set_error (error, COGL_WINSYS_ERROR,
-                       COGL_WINSYS_ERROR_INIT,
-                       "The GL driver was requested but SDL is using GLES");
+          _cogl_set_error (error, COGL_WINSYS_ERROR,
+                           COGL_WINSYS_ERROR_INIT,
+                           "The GL driver was requested but SDL is using GLES");
+          goto error;
+        }
+
+      if (display->renderer->driver == COGL_DRIVER_GL3 &&
+          gl_version[0] < '3')
+        {
+          _cogl_set_error (error, COGL_WINSYS_ERROR,
+                           COGL_WINSYS_ERROR_INIT,
+                           "The GL3 driver was requested but SDL is using "
+                           "GL %c", gl_version[0]);
           goto error;
         }
       break;
@@ -209,10 +236,10 @@ _cogl_winsys_display_setup (CoglDisplay *display,
     case COGL_DRIVER_GLES2:
       if (!g_str_has_prefix (gl_version, "OpenGL ES 2"))
         {
-          g_set_error (error, COGL_WINSYS_ERROR,
-                       COGL_WINSYS_ERROR_INIT,
-                       "The GLES2 driver was requested but SDL is "
-                       "not using GLES2");
+          _cogl_set_error (error, COGL_WINSYS_ERROR,
+                           COGL_WINSYS_ERROR_INIT,
+                           "The GLES2 driver was requested but SDL is "
+                           "not using GLES2");
           goto error;
         }
       break;
@@ -220,10 +247,10 @@ _cogl_winsys_display_setup (CoglDisplay *display,
     case COGL_DRIVER_GLES1:
       if (!g_str_has_prefix (gl_version, "OpenGL ES 1"))
         {
-          g_set_error (error, COGL_WINSYS_ERROR,
-                       COGL_WINSYS_ERROR_INIT,
-                       "The GLES1 driver was requested but SDL is "
-                       "not using GLES1");
+          _cogl_set_error (error, COGL_WINSYS_ERROR,
+                           COGL_WINSYS_ERROR_INIT,
+                           "The GLES1 driver was requested but SDL is "
+                           "not using GLES1");
           goto error;
         }
       break;
@@ -239,8 +266,49 @@ error:
   return FALSE;
 }
 
+static CoglFilterReturn
+sdl_event_filter_cb (SDL_Event *event, void *data)
+{
+  if (event->type == SDL_WINDOWEVENT &&
+      event->window.event == SDL_WINDOWEVENT_SIZE_CHANGED)
+    {
+      CoglContext *context = data;
+      CoglDisplay *display = context->display;
+      CoglDisplaySdl2 *sdl_display = display->winsys;
+      float width = event->window.data1;
+      float height = event->window.data2;
+      CoglFramebuffer *framebuffer;
+      CoglOnscreenSdl2 *sdl_onscreen;
+      SDL_Window *window;
+
+      window = SDL_GetWindowFromID (event->window.windowID);
+
+      if (window == NULL)
+        return COGL_FILTER_CONTINUE;
+
+      framebuffer = SDL_GetWindowData (window, COGL_SDL_WINDOW_DATA_KEY);
+
+      if (framebuffer == NULL || framebuffer->context != context)
+        return COGL_FILTER_CONTINUE;
+
+      _cogl_framebuffer_winsys_update_size (framebuffer, width, height);
+
+      sdl_onscreen = COGL_ONSCREEN (framebuffer)->winsys;
+
+      /* We only want to notify that a resize happened when the
+         application calls cogl_context_dispatch so instead of immediately
+         notifying we'll set a flag to remember to notify later */
+      sdl_display->pending_resize_notify = TRUE;
+      sdl_onscreen->pending_resize_notify = TRUE;
+
+      return COGL_FILTER_CONTINUE;
+    }
+
+  return COGL_FILTER_CONTINUE;
+}
+
 static CoglBool
-_cogl_winsys_context_init (CoglContext *context, GError **error)
+_cogl_winsys_context_init (CoglContext *context, CoglError **error)
 {
   CoglRenderer *renderer = context->display->renderer;
 
@@ -258,12 +326,23 @@ _cogl_winsys_context_init (CoglContext *context, GError **error)
                     COGL_WINSYS_FEATURE_SWAP_REGION_THROTTLE,
                     TRUE);
 
+  _cogl_renderer_add_native_filter (renderer,
+                                    (CoglNativeFilterFunc) sdl_event_filter_cb,
+                                    context);
+
   return TRUE;
 }
 
 static void
 _cogl_winsys_context_deinit (CoglContext *context)
 {
+  CoglRenderer *renderer = context->display->renderer;
+
+  _cogl_renderer_remove_native_filter (renderer,
+                                       (CoglNativeFilterFunc)
+                                       sdl_event_filter_cb,
+                                       context);
+
   g_free (context->winsys);
 }
 
@@ -309,7 +388,18 @@ _cogl_winsys_onscreen_deinit (CoglOnscreen *onscreen)
       CoglContextSdl2 *sdl_context = context->winsys;
 
       if (sdl_context->current_window == sdl_onscreen->window)
-        sdl_context->current_window = NULL;
+        {
+          CoglDisplaySdl2 *sdl_display = context->display->winsys;
+
+          /* SDL explicitly unbinds the context when the currently
+           * bound window is destroyed. Cogl always needs a context
+           * bound so that for example it can create texture resources
+           * at any time even without flushing a framebuffer.
+           * Therefore we'll bind the dummy window. */
+          SDL_GL_MakeCurrent (sdl_display->dummy_window,
+                              sdl_display->context);
+          sdl_context->current_window = sdl_display->dummy_window;
+        }
 
       SDL_DestroyWindow (sdl_onscreen->window);
       sdl_onscreen->window = NULL;
@@ -321,31 +411,40 @@ _cogl_winsys_onscreen_deinit (CoglOnscreen *onscreen)
 
 static CoglBool
 _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
-                            GError **error)
+                            CoglError **error)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   CoglOnscreenSdl2 *sdl_onscreen;
   SDL_Window *window;
   int width, height;
+  SDL_WindowFlags flags;
 
   width = cogl_framebuffer_get_width (framebuffer);
   height = cogl_framebuffer_get_height (framebuffer);
 
+  flags = SDL_WINDOW_OPENGL | SDL_WINDOW_HIDDEN;
+
+  /* The resizable property on SDL window apparently can only be set
+   * on creation */
+  if (onscreen->resizable)
+    flags |= SDL_WINDOW_RESIZABLE;
+
   window = SDL_CreateWindow ("" /* title */,
                              0, 0, /* x/y */
                              width, height,
-                             SDL_WINDOW_OPENGL |
-                             SDL_WINDOW_HIDDEN);
+                             flags);
 
   if (window == NULL)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
-                   COGL_WINSYS_ERROR_CREATE_ONSCREEN,
-                   "SDL_CreateWindow failed: %s",
-                   SDL_GetError ());
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
+                       COGL_WINSYS_ERROR_CREATE_ONSCREEN,
+                       "SDL_CreateWindow failed: %s",
+                       SDL_GetError ());
       return FALSE;
     }
 
+  SDL_SetWindowData (window, COGL_SDL_WINDOW_DATA_KEY, onscreen);
+
   onscreen->winsys = g_slice_new (CoglOnscreenSdl2);
   sdl_onscreen = onscreen->winsys;
   sdl_onscreen->window = window;
@@ -387,6 +486,57 @@ _cogl_winsys_onscreen_set_visibility (CoglOnscreen *onscreen,
     SDL_HideWindow (sdl_onscreen->window);
 }
 
+static void
+flush_pending_notifications_cb (void *data,
+                                void *user_data)
+{
+  CoglFramebuffer *framebuffer = data;
+
+  if (framebuffer->type == COGL_FRAMEBUFFER_TYPE_ONSCREEN)
+    {
+      CoglOnscreen *onscreen = COGL_ONSCREEN (framebuffer);
+      CoglOnscreenSdl2 *sdl_onscreen = onscreen->winsys;
+
+      if (sdl_onscreen->pending_resize_notify)
+        {
+          _cogl_onscreen_notify_resize (onscreen);
+          sdl_onscreen->pending_resize_notify = FALSE;
+        }
+    }
+}
+
+static void
+_cogl_winsys_poll_dispatch (CoglContext *context,
+                            const CoglPollFD *poll_fds,
+                            int n_poll_fds)
+{
+  CoglDisplay *display = context->display;
+  CoglDisplaySdl2 *sdl_display = display->winsys;
+
+  if (sdl_display->pending_resize_notify)
+    {
+      g_list_foreach (context->framebuffers,
+                      flush_pending_notifications_cb,
+                      NULL);
+      sdl_display->pending_resize_notify = FALSE;
+    }
+}
+
+SDL_Window *
+cogl_sdl_onscreen_get_window (CoglOnscreen *onscreen)
+{
+  CoglOnscreenSdl2 *sdl_onscreen;
+
+  _COGL_RETURN_VAL_IF_FAIL (cogl_is_onscreen (onscreen), NULL);
+
+  if (!cogl_framebuffer_allocate (COGL_FRAMEBUFFER (onscreen), NULL))
+    return NULL;
+
+  sdl_onscreen = onscreen->winsys;
+
+  return sdl_onscreen->window;
+}
+
 const CoglWinsysVtable *
 _cogl_winsys_sdl_get_vtable (void)
 {
@@ -419,6 +569,8 @@ _cogl_winsys_sdl_get_vtable (void)
         _cogl_winsys_onscreen_update_swap_throttled;
       vtable.onscreen_set_visibility = _cogl_winsys_onscreen_set_visibility;
 
+      vtable.poll_dispatch = _cogl_winsys_poll_dispatch;
+
       vtable_inited = TRUE;
     }
 
