@@ -9,9 +9,9 @@
    within Cogl. Cogl doesn't export the symbols for this data type so
    we just directly include the source instead */
 
+#define _COGL_IN_TEST_BITMASK
 #include <cogl/cogl-bitmask.h>
 #include <cogl/cogl-bitmask.c>
-#define _COGL_IN_TEST_BITMASK
 #include <cogl/cogl-util.c>
 
 typedef struct
@@ -20,7 +20,7 @@ typedef struct
   int *bits;
 } CheckData;
 
-static gboolean
+static CoglBool
 check_bit (int bit_num, void *user_data)
 {
   CheckData *data = user_data;
