@@ -277,7 +277,7 @@ test_gles2_read_pixels (void)
   CoglGLES2Context *gles2_ctx;
   const CoglGLES2Vtable *gles2;
   CoglError *error = NULL;
-  GLubyte pixel[3];
+  GLubyte pixel[4];
   GLuint fbo_handle;
 
   create_gles2_context (&offscreen_texture,
@@ -299,11 +299,9 @@ test_gles2_read_pixels (void)
 
   gles2->glClearColor (1, 0, 0, 1);
   gles2->glClear (GL_COLOR_BUFFER_BIT);
-  gles2->glReadPixels (0, 0, 1, 1, GL_RGB, GL_UNSIGNED_BYTE, &pixel);
+  gles2->glReadPixels (0, 0, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
 
-  g_assert (pixel[0] == 0xff);
-  g_assert (pixel[1] == 0);
-  g_assert (pixel[2] == 0);
+  test_utils_compare_pixel (pixel, 0xff0000ff);
 
   fbo_handle = create_gles2_framebuffer (gles2, 256, 256);
 
@@ -311,21 +309,17 @@ test_gles2_read_pixels (void)
 
   gles2->glClearColor (0, 1, 0, 1);
   gles2->glClear (GL_COLOR_BUFFER_BIT);
-  gles2->glReadPixels (0, 0, 1, 1, GL_RGB, GL_UNSIGNED_BYTE, &pixel);
+  gles2->glReadPixels (0, 0, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
 
-  g_assert (pixel[0] == 0);
-  g_assert (pixel[1] == 0xff);
-  g_assert (pixel[2] == 0);
+  test_utils_compare_pixel (pixel, 0x00ff00ff);
 
   gles2->glBindFramebuffer (GL_FRAMEBUFFER, 0);
 
   gles2->glClearColor (0, 1, 1, 1);
   gles2->glClear (GL_COLOR_BUFFER_BIT);
-  gles2->glReadPixels (0, 0, 1, 1, GL_RGB, GL_UNSIGNED_BYTE, &pixel);
+  gles2->glReadPixels (0, 0, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
 
-  g_assert (pixel[0] == 0);
-  g_assert (pixel[1] == 0xff);
-  g_assert (pixel[2] == 0xff);
+  test_utils_compare_pixel (pixel, 0x00ffffff);
 
   cogl_pop_gles2_context (test_ctx);
 
@@ -341,11 +335,9 @@ test_gles2_read_pixels (void)
       g_error ("Failed to push gles2 context: %s\n", error->message);
     }
 
-  gles2->glReadPixels (0, 0, 1, 1, GL_RGB, GL_UNSIGNED_BYTE, &pixel);
+  gles2->glReadPixels (0, 0, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
 
-  g_assert (pixel[0] == 0);
-  g_assert (pixel[1] == 0xff);
-  g_assert (pixel[2] == 0xff);
+  test_utils_compare_pixel (pixel, 0x00ffffff);
 
   cogl_pop_gles2_context (test_ctx);
 
@@ -363,11 +355,9 @@ test_gles2_read_pixels (void)
       g_error ("Failed to push gles2 context: %s\n", error->message);
     }
 
-  gles2->glReadPixels (0, 0, 1, 1, GL_RGB, GL_UNSIGNED_BYTE, &pixel);
+  gles2->glReadPixels (0, 0, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
 
-  g_assert (pixel[0] == 0xff);
-  g_assert (pixel[1] == 0xff);
-  g_assert (pixel[2] == 0xff);
+  test_utils_compare_pixel (pixel, 0xffffffff);
 
   cogl_pop_gles2_context (test_ctx);
 }
